# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the ios_acls module
"""


class AclsArgs(object):  # pylint: disable=R0903
    """The arg spec for the ios_acls module"""

    argument_spec = {
        "config": {
            "elements": "dict",
            "options": {
                "acls": {
                    "elements": "dict",
                    "options": {
                        "aces": {
                            "elements": "dict",
                            "options": {
                                "destination": {
                                    "options": {
                                        "address": {"type": "str"},
                                        "any": {"type": "bool"},
                                        "host": {"type": "str"},
                                        "object_group": {"type": "str"},
                                        "port_protocol": {
                                            "options": {
                                                "eq": {"type": "str"},
                                                "gt": {"type": "str"},
                                                "lt": {"type": "str"},
                                                "neq": {"type": "str"},
                                                "range": {
                                                    "options": {
                                                        "end": {"type": "int"},
                                                        "start": {"type": "int"},
                                                    },
                                                    "type": "dict",
                                                },
                                            },
                                            "type": "dict",
                                        },
                                        "wildcard_bits": {"type": "str"},
                                    },
                                    "type": "dict",
                                },
                                "dscp": {"type": "str"},
                                "evaluate": {"type": "str"},
                                "fragments": {"type": "str"},
                                "enable_fragments": {"type": "bool"},
                                "grant": {"choices": ["permit", "deny"], "type": "str"},
                                "log": {
                                    "options": {
                                        "set": {"type": "bool"},
                                        "user_cookie": {"type": "str"},
                                    },
                                    "type": "dict",
                                },
                                "log_input": {
                                    "options": {
                                        "set": {"type": "bool"},
                                        "user_cookie": {"type": "str"},
                                    },
                                    "type": "dict",
                                },
                                "option": {
                                    "options": {
                                        "add_ext": {"type": "bool"},
                                        "any_options": {"type": "bool"},
                                        "com_security": {"type": "bool"},
                                        "dps": {"type": "bool"},
                                        "encode": {"type": "bool"},
                                        "eool": {"type": "bool"},
                                        "ext_ip": {"type": "bool"},
                                        "ext_security": {"type": "bool"},
                                        "finn": {"type": "bool"},
                                        "imitd": {"type": "bool"},
                                        "lsr": {"type": "bool"},
                                        "mtup": {"type": "bool"},
                                        "mtur": {"type": "bool"},
                                        "no_op": {"type": "bool"},
                                        "nsapa": {"type": "bool"},
                                        "record_route": {"type": "bool"},
                                        "router_alert": {"type": "bool"},
                                        "sdb": {"type": "bool"},
                                        "security": {"type": "bool"},
                                        "ssr": {"type": "bool"},
                                        "stream_id": {"type": "bool"},
                                        "timestamp": {"type": "bool"},
                                        "traceroute": {"type": "bool"},
                                        "ump": {"type": "bool"},
                                        "visa": {"type": "bool"},
                                        "zsu": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "precedence": {"type": "str"},
                                "protocol": {"type": "str"},
                                "protocol_options": {
                                    "options": {
                                        "ahp": {"type": "bool"},
                                        "eigrp": {"type": "bool"},
                                        "esp": {"type": "bool"},
                                        "gre": {"type": "bool"},
                                        "hbh": {"type": "bool"},
                                        "icmp": {
                                            "options": {
                                                "administratively_prohibited": {"type": "bool"},
                                                "alternate_address": {"type": "bool"},
                                                "conversion_error": {"type": "bool"},
                                                "dod_host_prohibited": {"type": "bool"},
                                                "dod_net_prohibited": {"type": "bool"},
                                                "echo": {"type": "bool"},
                                                "echo_reply": {"type": "bool"},
                                                "general_parameter_problem": {"type": "bool"},
                                                "host_isolated": {"type": "bool"},
                                                "host_precedence_unreachable": {"type": "bool"},
                                                "host_redirect": {"type": "bool"},
                                                "host_tos_redirect": {"type": "bool"},
                                                "host_tos_unreachable": {"type": "bool"},
                                                "host_unknown": {"type": "bool"},
                                                "host_unreachable": {"type": "bool"},
                                                "information_reply": {"type": "bool"},
                                                "information_request": {"type": "bool"},
                                                "mask_reply": {"type": "bool"},
                                                "mask_request": {"type": "bool"},
                                                "mobile_redirect": {"type": "bool"},
                                                "net_redirect": {"type": "bool"},
                                                "net_tos_redirect": {"type": "bool"},
                                                "net_tos_unreachable": {"type": "bool"},
                                                "net_unreachable": {"type": "bool"},
                                                "network_unknown": {"type": "bool"},
                                                "no_room_for_option": {"type": "bool"},
                                                "option_missing": {"type": "bool"},
                                                "packet_too_big": {"type": "bool"},
                                                "parameter_problem": {"type": "bool"},
                                                "port_unreachable": {"type": "bool"},
                                                "precedence_unreachable": {"type": "bool"},
                                                "protocol_unreachable": {"type": "bool"},
                                                "reassembly_timeout": {"type": "bool"},
                                                "redirect": {"type": "bool"},
                                                "router_advertisement": {"type": "bool"},
                                                "router_solicitation": {"type": "bool"},
                                                "source_quench": {"type": "bool"},
                                                "source_route_failed": {"type": "bool"},
                                                "time_exceeded": {"type": "bool"},
                                                "timestamp_reply": {"type": "bool"},
                                                "timestamp_request": {"type": "bool"},
                                                "traceroute": {"type": "bool"},
                                                "ttl_exceeded": {"type": "bool"},
                                                "unreachable": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                        "igmp": {
                                            "options": {
                                                "dvmrp": {"type": "bool"},
                                                "host_query": {"type": "bool"},
                                                "mtrace_resp": {"type": "bool"},
                                                "mtrace_route": {"type": "bool"},
                                                "pim": {"type": "bool"},
                                                "trace": {"type": "bool"},
                                                "v1host_report": {"type": "bool"},
                                                "v2host_report": {"type": "bool"},
                                                "v2leave_group": {"type": "bool"},
                                                "v3host_report": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                        "ip": {"type": "bool"},
                                        "ipinip": {"type": "bool"},
                                        "ipv6": {"type": "bool"},
                                        "nos": {"type": "bool"},
                                        "ospf": {"type": "bool"},
                                        "pcp": {"type": "bool"},
                                        "pim": {"type": "bool"},
                                        "protocol_number": {"type": "int"},
                                        "sctp": {"type": "bool"},
                                        "tcp": {
                                            "options": {
                                                "ack": {"type": "bool"},
                                                "established": {"type": "bool"},
                                                "fin": {"type": "bool"},
                                                "psh": {"type": "bool"},
                                                "rst": {"type": "bool"},
                                                "syn": {"type": "bool"},
                                                "urg": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                        "udp": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "remarks": {"elements": "str", "type": "list"},
                                "sequence": {"type": "int"},
                                "source": {
                                    "options": {
                                        "address": {"type": "str"},
                                        "any": {"type": "bool"},
                                        "host": {"type": "str"},
                                        "object_group": {"type": "str"},
                                        "port_protocol": {
                                            "options": {
                                                "eq": {"type": "str"},
                                                "gt": {"type": "str"},
                                                "lt": {"type": "str"},
                                                "neq": {"type": "str"},
                                                "range": {
                                                    "options": {
                                                        "end": {"type": "int"},
                                                        "start": {"type": "int"},
                                                    },
                                                    "type": "dict",
                                                },
                                            },
                                            "type": "dict",
                                        },
                                        "wildcard_bits": {"type": "str"},
                                    },
                                    "type": "dict",
                                },
                                "time_range": {"type": "str"},
                                "tos": {
                                    "options": {
                                        "max_reliability": {"type": "bool"},
                                        "max_throughput": {"type": "bool"},
                                        "min_delay": {"type": "bool"},
                                        "min_monetary_cost": {"type": "bool"},
                                        "normal": {"type": "bool"},
                                        "service_value": {"type": "int"},
                                    },
                                    "type": "dict",
                                },
                                "ttl": {
                                    "options": {
                                        "eq": {"type": "int"},
                                        "gt": {"type": "int"},
                                        "lt": {"type": "int"},
                                        "neq": {"type": "int"},
                                        "range": {
                                            "options": {
                                                "end": {"type": "int"},
                                                "start": {"type": "int"},
                                            },
                                            "type": "dict",
                                        },
                                    },
                                    "type": "dict",
                                },
                            },
                            "type": "list",
                        },
                        "acl_type": {"choices": ["extended", "standard"], "type": "str"},
                        "name": {"required": True, "type": "str"},
                    },
                    "type": "list",
                },
                "afi": {"choices": ["ipv4", "ipv6"], "required": True, "type": "str"},
            },
            "type": "list",
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "rendered",
                "parsed",
            ],
            "default": "merged",
            "type": "str",
        },
    }  # pylint: disable=C0301



# -*- coding: utf-8 -*-
# Copyright (c) 2014, Brian Coca, Josh Drake, et al
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
    author: Unknown (!UNKNOWN)
    name: redis
    short_description: Use Redis DB for cache
    description:
        - This cache uses JSON formatted, per host records saved in Redis.
    requirements:
      - redis>=2.4.5 (python lib)
    options:
      _uri:
        description:
          - A colon separated string of connection information for Redis.
          - The format is V(host:port:db:password), for example V(localhost:6379:0:changeme).
          - To use encryption in transit, prefix the connection with V(tls://), as in V(tls://localhost:6379:0:changeme).
          - To use redis sentinel, use separator V(;), for example V(localhost:26379;localhost:26379;0:changeme). Requires redis>=2.9.0.
        required: true
        env:
          - name: ANSIBLE_CACHE_PLUGIN_CONNECTION
        ini:
          - key: fact_caching_connection
            section: defaults
      _prefix:
        description: User defined prefix to use when creating the DB entries
        default: ansible_facts
        env:
          - name: ANSIBLE_CACHE_PLUGIN_PREFIX
        ini:
          - key: fact_caching_prefix
            section: defaults
      _keyset_name:
        description: User defined name for cache keyset name.
        default: ansible_cache_keys
        env:
          - name: ANSIBLE_CACHE_REDIS_KEYSET_NAME
        ini:
          - key: fact_caching_redis_keyset_name
            section: defaults
        version_added: 1.3.0
      _sentinel_service_name:
        description: The redis sentinel service name (or referenced as cluster name).
        env:
          - name: ANSIBLE_CACHE_REDIS_SENTINEL
        ini:
          - key: fact_caching_redis_sentinel
            section: defaults
        version_added: 1.3.0
      _timeout:
        default: 86400
        description: Expiration timeout in seconds for the cache plugin data. Set to 0 to never expire
        env:
          - name: ANSIBLE_CACHE_PLUGIN_TIMEOUT
        ini:
          - key: fact_caching_timeout
            section: defaults
        type: integer
'''

import re
import time
import json

from ansible.errors import AnsibleError
from ansible.module_utils.common.text.converters import to_native
from ansible.parsing.ajson import AnsibleJSONEncoder, AnsibleJSONDecoder
from ansible.plugins.cache import BaseCacheModule
from ansible.utils.display import Display

try:
    from redis import StrictRedis, VERSION
    HAS_REDIS = True
except ImportError:
    HAS_REDIS = False

display = Display()


class CacheModule(BaseCacheModule):
    """
    A caching module backed by redis.

    Keys are maintained in a zset with their score being the timestamp
    when they are inserted. This allows for the usage of 'zremrangebyscore'
    to expire keys. This mechanism is used or a pattern matched 'scan' for
    performance.
    """
    _sentinel_service_name = None
    re_url_conn = re.compile(r'^([^:]+|\[[^]]+\]):(\d+):(\d+)(?::(.*))?$')
    re_sent_conn = re.compile(r'^(.*):(\d+)$')

    def __init__(self, *args, **kwargs):
        uri = ''

        super(CacheModule, self).__init__(*args, **kwargs)
        if self.get_option('_uri'):
            uri = self.get_option('_uri')
        self._timeout = float(self.get_option('_timeout'))
        self._prefix = self.get_option('_prefix')
        self._keys_set = self.get_option('_keyset_name')
        self._sentinel_service_name = self.get_option('_sentinel_service_name')

        if not HAS_REDIS:
            raise AnsibleError("The 'redis' python module (version 2.4.5 or newer) is required for the redis fact cache, 'pip install redis'")

        self._cache = {}
        kw = {}

        # tls connection
        tlsprefix = 'tls://'
        if uri.startswith(tlsprefix):
            kw['ssl'] = True
            uri = uri[len(tlsprefix):]

        # redis sentinel connection
        if self._sentinel_service_name:
            self._db = self._get_sentinel_connection(uri, kw)
        # normal connection
        else:
            connection = self._parse_connection(self.re_url_conn, uri)
            self._db = StrictRedis(*connection, **kw)

        display.vv('Redis connection: %s' % self._db)

    @staticmethod
    def _parse_connection(re_patt, uri):
        match = re_patt.match(uri)
        if not match:
            raise AnsibleError("Unable to parse connection string")
        return match.groups()

    def _get_sentinel_connection(self, uri, kw):
        """
        get sentinel connection details from _uri
        """
        try:
            from redis.sentinel import Sentinel
        except ImportError:
            raise AnsibleError("The 'redis' python module (version 2.9.0 or newer) is required to use redis sentinel.")

        if ';' not in uri:
            raise AnsibleError('_uri does not have sentinel syntax.')

        # format: "localhost:26379;localhost2:26379;0:changeme"
        connections = uri.split(';')
        connection_args = connections.pop(-1)
        if len(connection_args) > 0:  # handle if no db nr is given
            connection_args = connection_args.split(':')
            kw['db'] = connection_args.pop(0)
            try:
                kw['password'] = connection_args.pop(0)
            except IndexError:
                pass  # password is optional

        sentinels = [self._parse_connection(self.re_sent_conn, shost) for shost in connections]
        display.vv('\nUsing redis sentinels: %s' % sentinels)
        scon = Sentinel(sentinels, **kw)
        try:
            return scon.master_for(self._sentinel_service_name, socket_timeout=0.2)
        except Exception as exc:
            raise AnsibleError('Could not connect to redis sentinel: %s' % to_native(exc))

    def _make_key(self, key):
        return self._prefix + key

    def get(self, key):

        if key not in self._cache:
            value = self._db.get(self._make_key(key))
            # guard against the key not being removed from the zset;
            # this could happen in cases where the timeout value is changed
            # between invocations
            if value is None:
                self.delete(key)
                raise KeyError
            self._cache[key] = json.loads(value, cls=AnsibleJSONDecoder)

        return self._cache.get(key)

    def set(self, key, value):

        value2 = json.dumps(value, cls=AnsibleJSONEncoder, sort_keys=True, indent=4)
        if self._timeout > 0:  # a timeout of 0 is handled as meaning 'never expire'
            self._db.setex(self._make_key(key), int(self._timeout), value2)
        else:
            self._db.set(self._make_key(key), value2)

        if VERSION[0] == 2:
            self._db.zadd(self._keys_set, time.time(), key)
        else:
            self._db.zadd(self._keys_set, {key: time.time()})
        self._cache[key] = value

    def _expire_keys(self):
        if self._timeout > 0:
            expiry_age = time.time() - self._timeout
            self._db.zremrangebyscore(self._keys_set, 0, expiry_age)

    def keys(self):
        self._expire_keys()
        return self._db.zrange(self._keys_set, 0, -1)

    def contains(self, key):
        self._expire_keys()
        return (self._db.zrank(self._keys_set, key) is not None)

    def delete(self, key):
        if key in self._cache:
            del self._cache[key]
        self._db.delete(self._make_key(key))
        self._db.zrem(self._keys_set, key)

    def flush(self):
        for key in list(self.keys()):
            self.delete(key)

    def copy(self):
        # TODO: there is probably a better way to do this in redis
        ret = dict([(k, self.get(k)) for k in self.keys()])
        return ret

    def __getstate__(self):
        return dict()

    def __setstate__(self, data):
        self.__init__()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Anvitha Jain (@anvitha-jain) <anvjain@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_template_clone
short_description: Clone templates
description:
- Clone templates on Cisco ACI Multi-Site.
- Clones only template objects and not site objects.
author:
- Anvitha Jain (@anvitha-jain)
options:
  source_schema:
    description:
    - The name of the source_schema.
    type: str
  destination_schema:
    description:
    - The name of the destination_schema.
    type: str
  destination_tenant:
    description:
    - The name of the destination_schema.
    type: str
  source_template_name:
    description:
    - The name of the source template.
    type: str
  destination_template_name:
    description:
    - The name of the destination template.
    type: str
  destination_template_display_name:
    description:
    - The display name of the destination template.
    type: str
  state:
    description:
    - Use C(clone) for adding.
    type: str
    choices: [ clone ]
    default: clone
seealso:
- module: cisco.mso.mso_schema
- module: cisco.mso.mso_schema_clone
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Clone template in the same schema
  cisco.mso.mso_schema_template_clone:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    source_schema: Schema1
    destination_schema: Schema1
    destination_tenant: ansible_test
    source_template_name: Template1
    destination_template_name: Template1_clone
    destination_template_display_name: Template1_clone
    state: clone
  delegate_to: localhost

- name: Clone template to different schema
  cisco.mso.mso_schema_template_clone:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    source_schema: Schema1
    destination_schema: Schema2
    destination_tenant: ansible_test
    source_template_name: Template2
    destination_template_name: Cloned_template_1
    destination_template_display_name: Cloned_template_1
    state: clone
  delegate_to: localhost

- name: Clone template in the same schema but different tenant attached
  cisco.mso.mso_schema_template_clone:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    source_schema: Schema1
    destination_schema: Schema1
    destination_tenant: common
    source_template_name: Template1_clone
    destination_template_name: Template1_clone_2
    state: clone
  delegate_to: localhost
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.constants import NDO_4_UNIQUE_IDENTIFIERS
import json


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        source_schema=dict(type="str"),
        destination_schema=dict(type="str"),
        destination_tenant=dict(type="str"),
        source_template_name=dict(type="str"),
        destination_template_name=dict(type="str"),
        destination_template_display_name=dict(type="str"),
        state=dict(type="str", default="clone", choices=["clone"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "clone", ["source_schema", "source_template_name"]],
        ],
    )

    source_schema = module.params.get("source_schema")
    destination_schema = module.params.get("destination_schema")
    destination_tenant = module.params.get("destination_tenant")
    source_template_name = module.params.get("source_template_name")
    destination_template_name = module.params.get("destination_template_name")
    destination_template_display_name = module.params.get("destination_template_display_name")
    state = module.params.get("state")

    mso = MSOModule(module)

    source_schema_id = None
    destination_schema_id = None
    destination_tenant_id = None
    ops = []

    if destination_schema is None:
        destination_schema = source_schema

    if destination_template_name is None:
        destination_template_name = source_template_name

    if destination_template_display_name is None:
        destination_template_display_name = destination_template_name

    # Check if source and destination template are named differently if in same schema
    if source_schema == destination_schema:
        if source_template_name == destination_template_name:
            mso.fail_json(msg="Source and destination templates in the same schema cannot have same names.")

    # Get source schema id and destination schema id
    schema_summary = mso.query_objs("schemas/list-identity", key="schemas")

    for schema in schema_summary:
        if schema.get("displayName") == source_schema:
            source_schema_id = schema.get("id")

        if schema.get("displayName") == destination_schema:
            destination_schema_id = schema.get("id")
            for template in schema.get("templates"):
                if template.get("name") == destination_template_name:
                    mso.fail_json(msg="Template with the name '{0}' already exists. Please use another name.".format(destination_template_name))

    if source_schema_id is None:
        mso.fail_json(msg="Schema with the name '{0}' does not exist.".format(source_schema))
    elif destination_schema_id is None:
        mso.fail_json(msg="Schema with the name '{0}' does not exist.".format(destination_schema))

    # Get destination schema details before change
    destination_schema_path = "schemas/{0}".format(destination_schema_id)
    mso.existing = mso.query_obj(destination_schema_path, displayName=destination_schema)

    if state == "clone":
        # Get destination tenant id
        if destination_tenant is not None:
            destination_tenant_id = mso.lookup_tenant(destination_tenant)

        # Get source schema details
        source_schema_path = "schemas/{0}".format(source_schema_id)
        source_schema_obj = mso.query_obj(source_schema_path, displayName=source_schema)

        source_template_path = "/{0}/templates/{1}".format(source_schema_path, source_template_name)
        destination_template_path = "/{0}/templates/{1}".format(destination_schema_path, destination_template_name)

        source_templates = source_schema_obj.get("templates")
        new_template = None
        for template in source_templates:
            if template.get("name") == source_template_name:
                new_template = json.loads(json.dumps(template).replace(source_template_path, destination_template_path))
                new_template["name"] = destination_template_name
                new_template["displayName"] = destination_template_display_name
                if destination_tenant_id is not None:
                    new_template["tenantId"] = destination_tenant_id
                mso.delete_keys_from_dict(new_template, NDO_4_UNIQUE_IDENTIFIERS)
                break

        if new_template is None:
            mso.fail_json(msg="Source template with the name '{0}' does not exist.".format(source_template_name))

        new_template = mso.recursive_dict_from_ref(new_template)
        mso.previous = mso.existing

        ops.append(dict(op="add", path="/templates/-", value=new_template))
        if not module.check_mode:
            mso.request(destination_schema_path, method="PATCH", data=ops)

        mso.existing = mso.query_obj(destination_schema_path, displayName=destination_schema)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022, Tim Cragg (@timcragg)
# Copyright: (c) 2023, Akini Ross (@akinross)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_dhcp_relay
short_description: Manage DHCP relay policies.
description:
- Manage DHCP relay policy (dhcpRelayP) configuration on Cisco ACI fabrics.
options:
  tenant:
    description:
    - Name of an existing tenant.
    - When tenant is not provided the module will be applied to the global (infra) policy.
    type: str
  name:
    description:
    - Name of the DHCP relay policy
    type: str
    aliases: [ relay_policy ]
  description:
    description:
    - Description of the relay policy
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The C(tenant) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) modules can be used for this.
seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(dhcpRelayP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Add a new DHCP relay policy
  cisco.aci.aci_dhcp_relay:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: Auto-Demo
    name: my_dhcp_relay
    description: via Ansible
    state: present
  delegate_to: localhost

- name: Add a new global (infra) DHCP relay policy
  cisco.aci.aci_dhcp_relay:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: my_dhcp_relay
    description: via Ansible
    state: present
  delegate_to: localhost

- name: Remove a DHCP relay policy
  cisco.aci.aci_dhcp_relay:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: Auto-Demo
    name: my_dhcp_relay
    state: absent
  delegate_to: localhost

- name: Remove a global (infra) DHCP relay policy
  cisco.aci.aci_dhcp_relay:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: my_dhcp_relay
    state: absent
  delegate_to: localhost

- name: Query a DHCP relay policy
  cisco.aci.aci_dhcp_relay:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: Auto-Demo
    name: my_dhcp_relay
    state: query
  delegate_to: localhost
  register: query_result

- name: Query a global (infra) DHCP relay policy
  cisco.aci.aci_dhcp_relay:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: my_dhcp_relay
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all DHCP relay policies in a specific tenant
  cisco.aci.aci_dhcp_relay:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: Auto-Demo
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all DHCP relay policies
  cisco.aci.aci_dhcp_relay:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

"""

RETURN = r"""
   current:
     description: The existing configuration from the APIC after the module has finished
     returned: success
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production environment",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   error:
     description: The error information as returned from the APIC
     returned: failure
     type: dict
     sample:
       {
           "code": "122",
           "text": "unknown managed object class foo"
       }
   raw:
     description: The raw output returned by the APIC REST API (xml or json)
     returned: parse error
     type: str
     sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class "/></imdata>'
   sent:
     description: The actual/minimal configuration pushed to the APIC
     returned: info
     type: list
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment"
               }
           }
       }
   previous:
     description: The original configuration from the APIC before the module has started
     returned: info
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   proposed:
     description: The assembled configuration from the user-provided parameters
     returned: info
     type: dict
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment",
                   "name": "production"
               }
           }
       }
   filter_string:
     description: The filter string used for the request
     returned: failure or debug
     type: str
     sample: ?rsp-prop-include=config-only
   method:
     description: The HTTP method used for the request to the APIC
     returned: failure or debug
     type: str
     sample: POST
   response:
     description: The HTTP response from the APIC
     returned: failure or debug
     type: str
     sample: OK (30 bytes)
   status:
     description: The HTTP status from the APIC
     returned: failure or debug
     type: int
     sample: 200
   url:
     description: The HTTP url used for the request to the APIC
     returned: failure or debug
     type: str
     sample: https://10.11.12.13/api/mo/uni/tn-production.json
   """

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        name=dict(type="str", aliases=["relay_policy"]),
        description=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        tenant=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name"]],
            ["state", "present", ["name"]],
        ],
    )

    name = module.params.get("name")
    description = module.params.get("description")
    state = module.params.get("state")
    tenant = module.params.get("tenant")
    child_classes = ["dhcpRsProv"]

    if tenant is None:
        root_class = dict(
            aci_class="dhcpRelayP",
            aci_rn="infra/relayp-{0}".format(name),
            module_object=name,
            target_filter={"name": name},
        )
        subclass_1 = None
        owner = "infra"
    else:
        root_class = dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        )
        subclass_1 = dict(
            aci_class="dhcpRelayP",
            aci_rn="relayp-{0}".format(name),
            module_object=name,
            target_filter={"name": name},
        )
        owner = "tenant"

    aci = ACIModule(module)

    aci.construct_url(root_class=root_class, subclass_1=subclass_1, child_classes=child_classes)

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="dhcpRelayP",
            class_config=dict(name=name, descr=description, owner=owner),
        )

        aci.get_diff(aci_class="dhcpRelayP")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The cisco.ios_route_maps config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.route_maps import (
    Route_mapsTemplate,
)


class Route_maps(ResourceModule):
    """
    The cisco.ios_route_maps config class
    """

    parsers = ["continue_entry", "description"]

    def __init__(self, module):
        super(Route_maps, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="route_maps",
            tmplt=Route_mapsTemplate(),
        )

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """

        if self.want:
            wantd = {(entry["route_map"]): entry for entry in self.want}
        else:
            wantd = {}

        if self.have:
            haved = {(entry["route_map"]): entry for entry in self.have}
        else:
            haved = {}

        # Convert each of config list to dict
        for each in wantd, haved:
            self.list_to_dict(each)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            haved = {k: v for k, v in iteritems(haved) if k in wantd or not wantd}
            wantd = {}

        # remove superfluous config for overridden and deleted
        if self.state in ["overridden", "deleted"]:
            for k, have in iteritems(haved):
                if k not in wantd:
                    route_map_cmd = "no route-map {route_map}".format(**have)
                    self.commands.append(route_map_cmd)

        for k, want in iteritems(wantd):
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Route_maps network resource.
        """
        if want != have and self.state != "deleted":
            self.entries_compare(want, have)

    def entries_compare(self, want, have):
        if want.get("entries"):
            cmd_len = len(self.commands)
            if have.get("entries"):
                for k, v in iteritems(want["entries"]):
                    have_entry = have["entries"].pop(k, {})
                    if want["entries"][k] != have_entry:
                        # description gets merged with existing description, so explicit delete is required
                        # replaced and overridden state
                        if (
                            (self.state == "replaced" or self.state == "overridden")
                            and have_entry.get("description")
                            and have_entry.get("description")
                            != want["entries"][k].get("description")
                        ):
                            self.compare(parsers=["description"], want=dict(), have=have_entry)
                        self.compare(parsers=self.parsers, want=want["entries"][k], have=have_entry)
                        have_match = have_entry.get("match")
                        want_match = v.get("match")
                        if have_match and want_match:
                            self.list_type_compare("match", want=want_match, have=have_match)
                        elif not have_match and want_match:
                            self.list_type_compare("match", want=want_match, have=dict())
                        have_set = have_entry.get("set")
                        want_set = v.get("set")

                        if have_set and want_set:
                            self.list_type_compare("set", want=want_set, have=have_set)
                        elif not have_set and want_set:
                            self.list_type_compare("set", want=want_set, have=dict())

                    if cmd_len != len(self.commands):
                        route_map_cmd = "route-map {route_map}".format(**want)
                        if want["entries"][k].get("action"):
                            route_map_cmd += " {action}".format(**want["entries"][k])
                        if want["entries"][k].get("sequence"):
                            route_map_cmd += " {sequence}".format(**want["entries"][k])
                        self.commands.insert(cmd_len, route_map_cmd)
                        cmd_len = len(self.commands)
            else:
                for k, v in iteritems(want["entries"]):
                    self.compare(parsers=self.parsers, want=want["entries"][k], have=dict())
                    want_match = v.get("match")
                    if want_match:
                        self.list_type_compare("match", want=want_match, have=dict())
                    want_set = v.get("set")
                    if want_set:
                        self.list_type_compare("set", want=want_set, have=dict())
                    if cmd_len != len(self.commands):
                        route_map_cmd = "route-map {route_map}".format(**want)
                        if want["entries"][k].get("action"):
                            route_map_cmd += " {action}".format(**want["entries"][k])
                        if want["entries"][k].get("sequence"):
                            route_map_cmd += " {sequence}".format(**want["entries"][k])
                        self.commands.insert(cmd_len, route_map_cmd)
                        cmd_len = len(self.commands)

        if (self.state == "replaced" or self.state == "overridden") and have.get("entries"):
            cmd_len = len(self.commands)
            for k, v in iteritems(have["entries"]):
                route_map_cmd = "no route-map {route_map}".format(**have)
                if have["entries"][k].get("action"):
                    route_map_cmd += " {action}".format(**have["entries"][k])
                if have["entries"][k].get("sequence"):
                    route_map_cmd += " {sequence}".format(**have["entries"][k])
                self.commands.insert(cmd_len, route_map_cmd)

    def list_type_compare(self, compare_type, want, have):
        parsers = [
            "{0}".format(compare_type),
            "{0}.ip".format(compare_type),
            "{0}.ipv6".format(compare_type),
        ]
        for k, v in iteritems(want):
            have_v = have.pop(k, {})
            if v != have_v and k not in ["ip", "ipv6", "action", "sequence", "community"]:
                if have_v:
                    self.compare(
                        parsers=parsers,
                        want={compare_type: {k: v}},
                        have={compare_type: {k: have_v}},
                    )
                else:
                    self.compare(parsers=parsers, want={compare_type: {k: v}}, have=dict())

            if k in ["community"]:
                if have_v:
                    if have_v != v:
                        if self.state == "overridden" or self.state == "replaced":
                            self.compare(parsers=parsers, want={}, have={compare_type: {k: have_v}})
                        elif self.state == "merged":
                            for _key, _val in have_v.items():
                                if isinstance(_val, list):
                                    v[_key].extend(_val)
                                    v[_key] = list(set(v[_key]))
                                    v[_key].sort()

                        self.compare(
                            parsers=parsers,
                            want={compare_type: {k: v}},
                            have={compare_type: {k: have_v}},
                        )
                else:
                    self.compare(parsers=parsers, want={compare_type: {k: v}}, have=dict())

            if k in ["ip", "ipv6"]:
                for key, val in iteritems(v):
                    have_val = have_v.pop(key, {})
                    if val != have_val:
                        if have_val:
                            if self.state == "overridden" or self.state == "replaced":
                                self.compare(
                                    parsers=parsers,
                                    want=dict(),
                                    have={compare_type: {k: {key: have_val}}},
                                )
                            self.compare(
                                parsers=parsers,
                                want={compare_type: {k: {key: val}}},
                                have={compare_type: {k: {key: have_val}}},
                            )
                        else:
                            self.compare(
                                parsers=parsers,
                                want={compare_type: {k: {key: val}}},
                                have=dict(),
                            )
                if (self.state == "overridden" or self.state == "replaced") and have_v:
                    for key, val in iteritems(have_v):
                        self.compare(
                            parsers=parsers,
                            want=dict(),
                            have={compare_type: {k: {key: val}}},
                        )

        if have and (self.state == "replaced" or self.state == "overridden"):
            for k, v in iteritems(have):
                if k in ["ip", "ipv6"]:
                    for key, val in iteritems(v):
                        if key and val:
                            self.compare(
                                parsers=parsers,
                                want=dict(),
                                have={compare_type: {k: {key: val}}},
                            )
                else:
                    self.compare(parsers=parsers, want=dict(), have={compare_type: {k: v}})

    def list_to_dict(self, param):
        if param:

            def convert_to_dict(inner_match, key):
                temp = dict()
                for each in inner_match:
                    temp.update({key + "_" + str(each): each})
                return dict(sorted(temp.items(), key=lambda x: x[1]))

            for key, val in iteritems(param):
                temp_entries = dict()
                if val.get("entries"):
                    for every in val["entries"]:
                        match = every.get("match")
                        if match:
                            if match.get("as_path") and match.get("as_path").get("acls"):
                                match["as_path"]["acls"] = convert_to_dict(
                                    match["as_path"]["acls"],
                                    "acl",
                                )
                            if match.get("community") and match.get("community").get("name"):
                                match["community"]["name"] = convert_to_dict(
                                    match["community"]["name"],
                                    "name",
                                )
                            if match.get("extcommunity"):
                                match["extcommunity"] = convert_to_dict(
                                    match["extcommunity"],
                                    "num",
                                )
                            if match.get("interfaces"):
                                match["interfaces"] = convert_to_dict(
                                    match["interfaces"],
                                    "interface",
                                )
                            if match.get("ip"):
                                for each_ip_param in [
                                    "address",
                                    "flowspec",
                                    "next_hop",
                                    "redistribution_source",
                                    "route_source",
                                ]:
                                    if match["ip"].get(each_ip_param):
                                        if match["ip"][each_ip_param].get("acls"):
                                            match["ip"][each_ip_param]["acls"] = convert_to_dict(
                                                match["ip"][each_ip_param]["acls"],
                                                "acl",
                                            )
                                        elif match["ip"][each_ip_param].get("prefix_lists"):
                                            match["ip"][each_ip_param][
                                                "prefix_lists"
                                            ] = convert_to_dict(
                                                match["ip"][each_ip_param]["prefix_lists"],
                                                "prefix_list",
                                            )
                            if match.get("local_preference") and match.get("local_preference").get(
                                "value",
                            ):
                                match["local_preference"]["value"] = convert_to_dict(
                                    match["local_preference"]["value"],
                                    "value",
                                )
                            if match.get("mdt_group") and match.get("mdt_group").get("acls"):
                                match["mdt_group"]["acls"] = convert_to_dict(
                                    match["mdt_group"]["acls"],
                                    "acl",
                                )
                            if match.get("policy_lists"):
                                match["policy_lists"] = convert_to_dict(
                                    match["policy_lists"],
                                    "policy",
                                )
                            if match.get("security_group"):
                                for each_sg_param in ["source", "destination"]:
                                    if match.get("security_group").get(each_sg_param):
                                        match["security_group"][each_sg_param] = convert_to_dict(
                                            match["security_group"][each_sg_param],
                                            each_sg_param,
                                        )
                        set = every.get("set")
                        if set:
                            if set.get("interfaces"):
                                set["interfaces"] = convert_to_dict(set["interfaces"], "interface")
                            if set.get("as_path"):
                                _k = set.get("as_path").get("prepend")
                                if _k:
                                    if _k.get("as_number"):
                                        _k["as_number"] = " ".join(_k["as_number"])

                            if set.get("community"):
                                _k = set.get("community")
                                if _k and _k.get("number"):
                                    # asplain helper func
                                    def to_asplain(new_format):
                                        _int, _remainder = (int(i) for i in new_format.split(":"))
                                        return str(_int * 65536 + _remainder)

                                    # convert to asplain for correct sorting
                                    if ":" in _k["number"]:
                                        _k["number"] = list(
                                            map(to_asplain, _k["number"].split(" ")),
                                        )
                                    else:
                                        _k["number"] = _k["number"].split(" ")

                                    # sort the list to ensure idempotency
                                    _k["number"].sort()

                        action = every.get("action")
                        sequence = every.get("sequence")
                        temp_entries.update({action + "_" + str(sequence): every})
                    val["entries"] = temp_entries



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://wwww.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: cloudwatch_metric_alarm_info
version_added: 5.0.0
short_description: Gather information about the alarms for the specified metric
description:
  - Retrieves the alarms for the specified metric.
author:
  - Mandar Vijay Kulkarni (@mandar242)
options:
  alarm_names:
    description:
      - The name of the metric.
    required: false
    type: list
    elements: str
  alarm_name_prefix:
    description:
      - An alarm name prefix to retrieve information about alarms that have names that start with this prefix.
      - Can not be used with I(alarm_names).
    required: false
    type: str
  alarm_type:
    description:
      - Specify this to return metric alarms or composite alarms.
      - Module is defaulted to return metric alarms but can return composite alarms if I(alarm_type=CompositeAlarm).
    required: false
    type: str
    default: MetricAlarm
    choices: ['CompositeAlarm', 'MetricAlarm']
  children_of_alarm_name:
    description:
      - If specified returns information about the "children" alarms of the alarm name specified.
    required: false
    type: str
  parents_of_alarm_name:
    description:
      - If specified returns information about the "parent" alarms of the alarm name specified.
    required: false
    type: str
  state_value:
    description:
      - If specified returns information only about alarms that are currently in the particular state.
    required: false
    type: str
    choices: ['OK', 'ALARM', 'INSUFFICIENT_DATA']
  action_prefix:
    description:
      - This parameter can be used to filter the results of the operation to only those alarms that use a certain alarm action.
    required: false
    type: str

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: describe the metric alarm based on alarm names
  amazon.aws.cloudwatch_metric_alarm_info:
    alarm_names:
      - my-test-alarm-1
      - my-test-alarm-2

- name: describe the metric alarm based alarm names and state value
  amazon.aws.cloudwatch_metric_alarm_info:
    alarm_names:
      - my-test-alarm-1
      - my-test-alarm-2
    state_value: OK

- name: describe the metric alarm based alarm names prefix
  amazon.aws.cloudwatch_metric_alarm_info:
    alarm_name_prefix: my-test-
"""

RETURN = r"""
metric_alarms:
    description: The gathered information about specified metric alarms.
    returned: when success
    type: list
    elements: dict
    contains:
        alarm_name:
            description: Unique name for the alarm.
            returned: always
            type: str
        alarm_arn:
            description: The Amazon Resource Name (ARN) of the alarm.
            returned: always
            type: str
        alarm_description:
            description: The description of the alarm.
            returned: always
            type: str
        alarm_configuration_updated_timestamp:
            description: The time stamp of the last update to the alarm configuration.
            returned: always
            type: str
        actions_enabled:
            description: Indicates whether actions should be executed during any changes to the alarm state.
            returned: always
            type: bool
        ok_actions:
            description: The actions to execute when this alarm transitions to an OK state from any other state.
            returned: always
            type: list
            elements: str
        alarm_actions:
            description: The actions to execute when this alarm transitions to an ALARM state from any other state.
            returned: always
            type: list
            elements: str
        insufficient_data_actions:
            description: The actions to execute when this alarm transitions to an INSUFFICIENT_DATA state from any other state.
            returned: always
            type: list
            elements: str
        state_value:
            description: The state value for the alarm.
            returned: always
            type: str
        state_reason:
            description: An explanation for the alarm state, in text format.
            returned: always
            type: str
        state_reason_data:
            description: An explanation for the alarm state, in JSON format.
            returned: always
            type: str
        state_updated_timestamp:
            description: The time stamp of the last update to the alarm state.
            returned: always
            type: str
        metric_name:
            description: Name of the monitored metric (e.g. C(CPUUtilization)).
            returned: always
            type: str
        namespace:
            description:
                - Name of the appropriate namespace (C(AWS/EC2), C(System/Linux), etc.).
                - Determines the category it will appear under in CloudWatch.
            returned: always
            type: str
        statistic:
            description: The statistic for the metric associated with the alarm, other than percentile.
            returned: always
            type: str
        extended_statistic:
            description: The percentile statistic for the metric associated with the alarm.
            returned: always
            type: str
        dimensions:
            description: The dimensions for the metric.
            returned: always
            type: list
            elements: dict
            contains:
                name:
                    description: The name of the dimension.
                    returned: always
                    type: str
                value:
                    description: The value of the dimension.
                    returned: always
                    type: str
        period:
            description:
                - The length, in seconds, used each time the metric specified in MetricName is evaluated.
                - Valid values are 10, 30, and any multiple of 60.
            returned: always
            type: int
        unit:
            description: Unit used when storing the metric
            returned: always
            type: str
        evaluation_period:
            description: The number of periods over which data is compared to the specified threshold.
            returned: always
            type: int
        datapoints_to_alarm:
            description: The number of data points that must be breaching to trigger the alarm.
            returned: always
            type: int
        threshold:
            description: The value to compare with the specified statistic.
            returned: always
            type: float
        comparison_operator:
            description: The arithmetic operation to use when comparing the specified statistic and threshold.
            returned: always
            type: str
        treat_missing_data:
            description: Sets how alarm is to handle missing data points.
            returned: always
            type: str
        evaluate_low_sample_count_percentile:
            description:
              - Used only for alarms based on percentiles.
              - If I(ignore), the alarm state does not change during periods with too few data points to be statistically significant.
              - If I(evaluate) or this parameter is not used, the alarm is always evaluated and possibly changes state.
            returned: always
            type: str
        metrics:
            description: An array of MetricDataQuery structures, used in an alarm based on a metric math expression.
            returned: always
            type: list
            elements: dict
        threshold_metric_id:
            description: This is the ID of the ANOMALY_DETECTION_BAND function used as the threshold for the alarm.
            returned: always
            type: str
"""


try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


@AWSRetry.jittered_backoff(retries=10)
def _describe_alarms(connection, **params):
    paginator = connection.get_paginator("describe_alarms")
    return paginator.paginate(**params).build_full_result()


def describe_metric_alarms_info(connection, module):
    params = build_params(module)

    alarm_type_to_return = module.params.get("alarm_type")

    try:
        describe_metric_alarms_info_response = _describe_alarms(connection, **params)
        # describe_metric_alarms_info_response = describe_metric_alarms_info_response[alarm_type_to_return]
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to describe cloudwatch metric alarm")

    result = []

    if alarm_type_to_return == "CompositeAlarm":
        for response_list_item in describe_metric_alarms_info_response["CompositeAlarms"]:
            result.append(camel_dict_to_snake_dict(response_list_item))
        module.exit_json(composite_alarms=result)

    for response_list_item in describe_metric_alarms_info_response["MetricAlarms"]:
        result.append(camel_dict_to_snake_dict(response_list_item))

    module.exit_json(metric_alarms=result)


def build_params(module):
    params = {}

    if module.params.get("alarm_names"):
        params["AlarmNames"] = module.params.get("alarm_names")

    if module.params.get("alarm_name_prefix"):
        params["AlarmNamePrefix"] = module.params.get("alarm_name_prefix")

    if module.params.get("children_of_alarm_name"):
        params["ChildrenOfAlarmName"] = module.params.get("children_of_alarm_name")

    if module.params.get("parents_of_alarm_name"):
        params["ParentsOfAlarmName"] = module.params.get("parents_of_alarm_name")

    if module.params.get("state_value"):
        params["StateValue"] = module.params.get("state_value")

    if module.params.get("action_prefix"):
        params["ActionPrefix"] = module.params.get("action_prefix")

    return params


def main():
    argument_spec = dict(
        alarm_names=dict(type="list", elements="str", required=False),
        alarm_name_prefix=dict(type="str", required=False),
        alarm_type=dict(type="str", choices=["CompositeAlarm", "MetricAlarm"], default="MetricAlarm", required=False),
        children_of_alarm_name=dict(type="str", required=False),
        parents_of_alarm_name=dict(type="str", required=False),
        state_value=dict(type="str", choices=["OK", "ALARM", "INSUFFICIENT_DATA"], required=False),
        action_prefix=dict(type="str", required=False),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec, mutually_exclusive=[["alarm_names", "alarm_name_prefix"]], supports_check_mode=True
    )

    try:
        connection = module.client("cloudwatch", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    describe_metric_alarms_info(connection, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_device_module_info
short_description: Information module for Network Device Module
description:
- Get all Network Device Module.
- Get Network Device Module by id.
- Returns Module info by 'module id'.
- Returns modules by specified device id.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceId:
    description:
    - DeviceId query parameter.
    type: str
  limit:
    description:
    - Limit query parameter.
    type: int
  offset:
    description:
    - Offset query parameter.
    type: int
  nameList:
    description:
    - NameList query parameter.
    elements: str
    type: list
  vendorEquipmentTypeList:
    description:
    - VendorEquipmentTypeList query parameter.
    elements: str
    type: list
  partNumberList:
    description:
    - PartNumberList query parameter.
    elements: str
    type: list
  operationalStateCodeList:
    description:
    - OperationalStateCodeList query parameter.
    elements: str
    type: list
  id:
    description:
    - Id path parameter. Module id.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetModuleInfoById
  description: Complete reference of the GetModuleInfoById API.
  link: https://developer.cisco.com/docs/dna-center/#!get-module-info-by-id
- name: Cisco DNA Center documentation for Devices GetModules
  description: Complete reference of the GetModules API.
  link: https://developer.cisco.com/docs/dna-center/#!get-modules
notes:
  - SDK Method used are
    devices.Devices.get_module_info_by_id,
    devices.Devices.get_modules,

  - Paths used are
    get /dna/intent/api/v1/network-device/module,
    get /dna/intent/api/v1/network-device/module/{id},

"""

EXAMPLES = r"""
- name: Get all Network Device Module
  cisco.dnac.network_device_module_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceId: string
    limit: 0
    offset: 0
    nameList: []
    vendorEquipmentTypeList: []
    partNumberList: []
    operationalStateCodeList: []
  register: result

- name: Get Network Device Module by id
  cisco.dnac.network_device_module_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "assemblyNumber": "string",
        "assemblyRevision": "string",
        "attributeInfo": {},
        "containmentEntity": "string",
        "description": "string",
        "entityPhysicalIndex": "string",
        "id": "string",
        "isFieldReplaceable": "string",
        "isReportingAlarmsAllowed": "string",
        "manufacturer": "string",
        "moduleIndex": 0,
        "name": "string",
        "operationalStateCode": "string",
        "partNumber": "string",
        "serialNumber": "string",
        "vendorEquipmentType": "string"
      },
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-

# Copyright (c) 2016, Peter Sagerson <psagers@ignorare.net>
# Copyright (c) 2016, Jiri Tyr <jiri.tyr@gmail.com>
# Copyright (c) 2017-2018 Keller Fuchs (@KellerFuchs) <kellerfuchs@hashbang.sh>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type

import re
import traceback
from ansible.module_utils.common.text.converters import to_native

try:
    import ldap
    import ldap.dn
    import ldap.filter
    import ldap.sasl

    HAS_LDAP = True

    SASCL_CLASS = {
        'gssapi': ldap.sasl.gssapi,
        'external': ldap.sasl.external,
    }
except ImportError:
    HAS_LDAP = False


def gen_specs(**specs):
    specs.update({
        'bind_dn': dict(),
        'bind_pw': dict(default='', no_log=True),
        'ca_path': dict(type='path'),
        'dn': dict(required=True),
        'referrals_chasing': dict(type='str', default='anonymous', choices=['disabled', 'anonymous']),
        'server_uri': dict(default='ldapi:///'),
        'start_tls': dict(default=False, type='bool'),
        'validate_certs': dict(default=True, type='bool'),
        'sasl_class': dict(choices=['external', 'gssapi'], default='external', type='str'),
        'xorder_discovery': dict(choices=['enable', 'auto', 'disable'], default='auto', type='str'),
        'client_cert': dict(default=None, type='path'),
        'client_key': dict(default=None, type='path'),
    })

    return specs


def ldap_required_together():
    return [['client_cert', 'client_key']]


class LdapGeneric(object):
    def __init__(self, module):
        # Shortcuts
        self.module = module
        self.bind_dn = self.module.params['bind_dn']
        self.bind_pw = self.module.params['bind_pw']
        self.ca_path = self.module.params['ca_path']
        self.referrals_chasing = self.module.params['referrals_chasing']
        self.server_uri = self.module.params['server_uri']
        self.start_tls = self.module.params['start_tls']
        self.verify_cert = self.module.params['validate_certs']
        self.sasl_class = self.module.params['sasl_class']
        self.xorder_discovery = self.module.params['xorder_discovery']
        self.client_cert = self.module.params['client_cert']
        self.client_key = self.module.params['client_key']

        # Establish connection
        self.connection = self._connect_to_ldap()

        if self.xorder_discovery == "enable" or (self.xorder_discovery == "auto" and not self._xorder_dn()):
            # Try to find the X_ORDERed version of the DN
            self.dn = self._find_dn()
        else:
            self.dn = self.module.params['dn']

    def fail(self, msg, exn):
        self.module.fail_json(
            msg=msg,
            details=to_native(exn),
            exception=traceback.format_exc()
        )

    def _find_dn(self):
        dn = self.module.params['dn']

        explode_dn = ldap.dn.explode_dn(dn)

        if len(explode_dn) > 1:
            try:
                escaped_value = ldap.filter.escape_filter_chars(explode_dn[0])
                filterstr = "(%s)" % escaped_value
                dns = self.connection.search_s(','.join(explode_dn[1:]),
                                               ldap.SCOPE_ONELEVEL, filterstr)
                if len(dns) == 1:
                    dn, dummy = dns[0]
            except Exception:
                pass

        return dn

    def _connect_to_ldap(self):
        if not self.verify_cert:
            ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)

        if self.ca_path:
            ldap.set_option(ldap.OPT_X_TLS_CACERTFILE, self.ca_path)

        if self.client_cert and self.client_key:
            ldap.set_option(ldap.OPT_X_TLS_CERTFILE, self.client_cert)
            ldap.set_option(ldap.OPT_X_TLS_KEYFILE, self.client_key)

        connection = ldap.initialize(self.server_uri)

        if self.referrals_chasing == 'disabled':
            # Switch off chasing of referrals (https://github.com/ansible-collections/community.general/issues/1067)
            connection.set_option(ldap.OPT_REFERRALS, 0)

        if self.start_tls:
            try:
                connection.start_tls_s()
            except ldap.LDAPError as e:
                self.fail("Cannot start TLS.", e)

        try:
            if self.bind_dn is not None:
                connection.simple_bind_s(self.bind_dn, self.bind_pw)
            else:
                klass = SASCL_CLASS.get(self.sasl_class, ldap.sasl.external)
                connection.sasl_interactive_bind_s('', klass())
        except ldap.LDAPError as e:
            self.fail("Cannot bind to the server.", e)

        return connection

    def _xorder_dn(self):
        # match X_ORDERed DNs
        regex = r".+\{\d+\}.+"
        explode_dn = ldap.dn.explode_dn(self.module.params['dn'])

        return re.match(regex, explode_dn[0]) is not None



# -*- coding: utf-8 -*-

# Copyright (c) 2016 Michael Gruener <michael.gruener@chaosmoon.net>
# Copyright (c) 2021 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type

from ansible.module_utils.common.text.converters import to_text
from ansible.module_utils.six import binary_type, PY3
from ansible.module_utils.six.moves.http_client import responses as http_responses


def format_http_status(status_code):
    expl = http_responses.get(status_code)
    if not expl:
        return str(status_code)
    return '%d %s' % (status_code, expl)


def format_error_problem(problem, subproblem_prefix=''):
    error_type = problem.get('type', 'about:blank')  # https://www.rfc-editor.org/rfc/rfc7807#section-3.1
    if 'title' in problem:
        msg = 'Error "{title}" ({type})'.format(
            type=error_type,
            title=problem['title'],
        )
    else:
        msg = 'Error {type}'.format(type=error_type)
    if 'detail' in problem:
        msg += ': "{detail}"'.format(detail=problem['detail'])
    subproblems = problem.get('subproblems')
    if subproblems is not None:
        msg = '{msg} Subproblems:'.format(msg=msg)
        for index, problem in enumerate(subproblems):
            index_str = '{prefix}{index}'.format(prefix=subproblem_prefix, index=index)
            msg = '{msg}\n({index}) {problem}'.format(
                msg=msg,
                index=index_str,
                problem=format_error_problem(problem, subproblem_prefix='{0}.'.format(index_str)),
            )
    return msg


class ModuleFailException(Exception):
    '''
    If raised, module.fail_json() will be called with the given parameters after cleanup.
    '''
    def __init__(self, msg, **args):
        super(ModuleFailException, self).__init__(self, msg)
        self.msg = msg
        self.module_fail_args = args

    def do_fail(self, module, **arguments):
        module.fail_json(msg=self.msg, other=self.module_fail_args, **arguments)


class ACMEProtocolException(ModuleFailException):
    def __init__(self, module, msg=None, info=None, response=None, content=None, content_json=None, extras=None):
        # Try to get hold of content, if response is given and content is not provided
        if content is None and content_json is None and response is not None:
            try:
                # In Python 2, reading from a closed response yields a TypeError.
                # In Python 3, read() simply returns ''
                if PY3 and response.closed:
                    raise TypeError
                content = response.read()
            except (AttributeError, TypeError):
                content = info.pop('body', None)

        # Make sure that content_json is None or a dictionary
        if content_json is not None and not isinstance(content_json, dict):
            if content is None and isinstance(content_json, binary_type):
                content = content_json
            content_json = None

        # Try to get hold of JSON decoded content, when content is given and JSON not provided
        if content_json is None and content is not None and module is not None:
            try:
                content_json = module.from_json(to_text(content))
            except Exception as e:
                pass

        extras = extras or dict()

        if msg is None:
            msg = 'ACME request failed'
        add_msg = ''

        if info is not None:
            url = info['url']
            code = info['status']
            extras['http_url'] = url
            extras['http_status'] = code
            if code is not None and code >= 400 and content_json is not None and 'type' in content_json:
                if 'status' in content_json and content_json['status'] != code:
                    code_msg = 'status {problem_code} (HTTP status: {http_code})'.format(
                        http_code=format_http_status(code), problem_code=content_json['status'])
                else:
                    code_msg = 'status {problem_code}'.format(problem_code=format_http_status(code))
                    if code == -1 and info.get('msg'):
                        code_msg = 'error: {msg}'.format(msg=info['msg'])
                subproblems = content_json.pop('subproblems', None)
                add_msg = ' {problem}.'.format(problem=format_error_problem(content_json))
                extras['problem'] = content_json
                extras['subproblems'] = subproblems or []
                if subproblems is not None:
                    add_msg = '{add_msg} Subproblems:'.format(add_msg=add_msg)
                    for index, problem in enumerate(subproblems):
                        add_msg = '{add_msg}\n({index}) {problem}.'.format(
                            add_msg=add_msg,
                            index=index,
                            problem=format_error_problem(problem, subproblem_prefix='{0}.'.format(index)),
                        )
            else:
                code_msg = 'HTTP status {code}'.format(code=format_http_status(code))
                if code == -1 and info.get('msg'):
                    code_msg = 'error: {msg}'.format(msg=info['msg'])
                if content_json is not None:
                    add_msg = ' The JSON error result: {content}'.format(content=content_json)
                elif content is not None:
                    add_msg = ' The raw error result: {content}'.format(content=to_text(content))
            msg = '{msg} for {url} with {code}'.format(msg=msg, url=url, code=code_msg)
        elif content_json is not None:
            add_msg = ' The JSON result: {content}'.format(content=content_json)
        elif content is not None:
            add_msg = ' The raw result: {content}'.format(content=to_text(content))

        super(ACMEProtocolException, self).__init__(
            '{msg}.{add_msg}'.format(msg=msg, add_msg=add_msg),
            **extras
        )
        self.problem = {}
        self.subproblems = []
        for k, v in extras.items():
            setattr(self, k, v)


class BackendException(ModuleFailException):
    pass


class NetworkException(ModuleFailException):
    pass


class KeyParsingError(ModuleFailException):
    pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2019, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: x509_crl
version_added: '1.0.0'
short_description: Generate Certificate Revocation Lists (CRLs)
description:
    - This module allows one to (re)generate or update Certificate Revocation Lists (CRLs).
    - Certificates on the revocation list can be either specified by serial number and (optionally) their issuer,
      or as a path to a certificate file in PEM format.
requirements:
    - cryptography >= 1.2
author:
    - Felix Fontein (@felixfontein)
extends_documentation_fragment:
    - ansible.builtin.files
    - community.crypto.attributes
    - community.crypto.attributes.files
    - community.crypto.name_encoding
attributes:
    check_mode:
        support: full
    diff_mode:
        support: full
    safe_file_operations:
        support: full
options:
    state:
        description:
            - Whether the CRL file should exist or not, taking action if the state is different from what is stated.
        type: str
        default: present
        choices: [ absent, present ]

    crl_mode:
        description:
            - Defines how to process entries of existing CRLs.
            - If set to V(generate), makes sure that the CRL has the exact set of revoked certificates
              as specified in O(revoked_certificates).
            - If set to V(update), makes sure that the CRL contains the revoked certificates from
              O(revoked_certificates), but can also contain other revoked certificates. If the CRL file
              already exists, all entries from the existing CRL will also be included in the new CRL.
              When using V(update), you might be interested in setting O(ignore_timestamps) to V(true).
            - The default value is V(generate).
            - This parameter was called O(mode) before community.crypto 2.13.0. It has been renamed to avoid
              a collision with the common O(mode) parameter for setting the CRL file's access mode.
        type: str
        # default: generate
        choices: [ generate, update ]
        version_added: 2.13.0
    mode:
        description:
            - This parameter has been renamed to O(crl_mode). The old name O(mode) is now deprecated and will
              be removed in community.crypto 3.0.0. Replace usage of this parameter with O(crl_mode).
            - Note that from community.crypto 3.0.0 on, O(mode) will be used for the CRL file's mode.
        type: str
        # default: generate
        choices: [ generate, update ]

    force:
        description:
            - Should the CRL be forced to be regenerated.
        type: bool
        default: false

    backup:
        description:
            - Create a backup file including a timestamp so you can get the original
              CRL back if you overwrote it with a new one by accident.
        type: bool
        default: false

    path:
        description:
            - Remote absolute path where the generated CRL file should be created or is already located.
        type: path
        required: true

    format:
        description:
            - Whether the CRL file should be in PEM or DER format.
            - If an existing CRL file does match everything but O(format), it will be converted to the correct format
              instead of regenerated.
        type: str
        choices: [pem, der]
        default: pem

    privatekey_path:
        description:
            - Path to the CA's private key to use when signing the CRL.
            - Either O(privatekey_path) or O(privatekey_content) must be specified if O(state) is V(present), but not both.
        type: path

    privatekey_content:
        description:
            - The content of the CA's private key to use when signing the CRL.
            - Either O(privatekey_path) or O(privatekey_content) must be specified if O(state) is V(present), but not both.
        type: str

    privatekey_passphrase:
        description:
            - The passphrase for the O(privatekey_path).
            - This is required if the private key is password protected.
        type: str

    issuer:
        description:
            - Key/value pairs that will be present in the issuer name field of the CRL.
            - If you need to specify more than one value with the same key, use a list as value.
            - If the order of the components is important, use O(issuer_ordered).
            - One of O(issuer) and O(issuer_ordered) is required if O(state) is V(present).
            - Mutually exclusive with O(issuer_ordered).
        type: dict
    issuer_ordered:
        description:
            - A list of dictionaries, where every dictionary must contain one key/value pair.
              This key/value pair will be present in the issuer name field of the CRL.
            - If you want to specify more than one value with the same key in a row, you can
              use a list as value.
            - One of O(issuer) and O(issuer_ordered) is required if O(state) is V(present).
            - Mutually exclusive with O(issuer).
        type: list
        elements: dict
        version_added: 2.0.0

    last_update:
        description:
            - The point in time from which this CRL can be trusted.
            - Time can be specified either as relative time or as absolute timestamp.
            - Time will always be interpreted as UTC.
            - Valid format is C([+-]timespec | ASN.1 TIME) where timespec can be an integer
              + C([w | d | h | m | s]) (for example V(+32w1d2h)).
            - Note that if using relative time this module is NOT idempotent, except when
              O(ignore_timestamps) is set to V(true).
        type: str
        default: "+0s"

    next_update:
        description:
            - "The absolute latest point in time by which this O(issuer) is expected to have issued
               another CRL. Many clients will treat a CRL as expired once O(next_update) occurs."
            - Time can be specified either as relative time or as absolute timestamp.
            - Time will always be interpreted as UTC.
            - Valid format is C([+-]timespec | ASN.1 TIME) where timespec can be an integer
              + C([w | d | h | m | s]) (for example V(+32w1d2h)).
            - Note that if using relative time this module is NOT idempotent, except when
              O(ignore_timestamps) is set to V(true).
            - Required if O(state) is V(present).
        type: str

    digest:
        description:
            - Digest algorithm to be used when signing the CRL.
        type: str
        default: sha256

    serial_numbers:
        description:
            - This option determines which values will be accepted for O(revoked_certificates[].serial_number).
            - If set to V(integer) (default), serial numbers are assumed to be integers, for example V(66223).
              (This example value is equivalent to the hex octet string V(01:02:AF).)
            - If set to V(hex-octets), serial numbers are assumed to be colon-separated hex octet strings,
              for example V(01:02:AF).
              (This example value is equivalent to the integer V(66223).)
        type: str
        choices:
            - integer
            - hex-octets
        default: integer
        version_added: 2.18.0

    revoked_certificates:
        description:
            - List of certificates to be revoked.
            - Required if O(state) is V(present).
        type: list
        elements: dict
        suboptions:
            path:
                description:
                    - Path to a certificate in PEM format.
                    - The serial number and issuer will be extracted from the certificate.
                    - Mutually exclusive with O(revoked_certificates[].content) and
                      O(revoked_certificates[].serial_number). One of these three options
                      must be specified.
                type: path
            content:
                description:
                    - Content of a certificate in PEM format.
                    - The serial number and issuer will be extracted from the certificate.
                    - Mutually exclusive with O(revoked_certificates[].path) and
                      O(revoked_certificates[].serial_number). One of these three options
                      must be specified.
                type: str
            serial_number:
                description:
                    - Serial number of the certificate.
                    - Mutually exclusive with O(revoked_certificates[].path) and
                      O(revoked_certificates[].content). One of these three options must
                      be specified.
                    - This option accepts integers or hex octet strings, depending on the value
                      of O(serial_numbers).
                    - If O(serial_numbers=integer), integers such as V(66223) must be provided.
                    - If O(serial_numbers=hex-octets), strings such as V(01:02:AF) must be provided.
                    - You can use the filters P(community.crypto.parse_serial#filter) and
                      P(community.crypto.to_serial#filter) to convert these two representations.
                type: raw
            revocation_date:
                description:
                    - The point in time the certificate was revoked.
                    - Time can be specified either as relative time or as absolute timestamp.
                    - Time will always be interpreted as UTC.
                    - Valid format is C([+-]timespec | ASN.1 TIME) where timespec can be an integer
                      + C([w | d | h | m | s]) (for example V(+32w1d2h)).
                    - Note that if using relative time this module is NOT idempotent, except when
                      O(ignore_timestamps) is set to V(true).
                type: str
                default: "+0s"
            issuer:
                description:
                    - The certificate's issuer.
                    - "Example: V(DNS:ca.example.org)"
                type: list
                elements: str
            issuer_critical:
                description:
                    - Whether the certificate issuer extension should be critical.
                type: bool
                default: false
            reason:
                description:
                    - The value for the revocation reason extension.
                type: str
                choices:
                    - unspecified
                    - key_compromise
                    - ca_compromise
                    - affiliation_changed
                    - superseded
                    - cessation_of_operation
                    - certificate_hold
                    - privilege_withdrawn
                    - aa_compromise
                    - remove_from_crl
            reason_critical:
                description:
                    - Whether the revocation reason extension should be critical.
                type: bool
                default: false
            invalidity_date:
                description:
                    - The point in time it was known/suspected that the private key was compromised
                      or that the certificate otherwise became invalid.
                    - Time can be specified either as relative time or as absolute timestamp.
                    - Time will always be interpreted as UTC.
                    - Valid format is C([+-]timespec | ASN.1 TIME) where timespec can be an integer
                      + C([w | d | h | m | s]) (for example V(+32w1d2h)).
                    - Note that if using relative time this module is NOT idempotent. This will NOT
                      change when O(ignore_timestamps) is set to V(true).
                type: str
            invalidity_date_critical:
                description:
                    - Whether the invalidity date extension should be critical.
                type: bool
                default: false

    ignore_timestamps:
        description:
            - Whether the timestamps O(last_update), O(next_update) and
              O(revoked_certificates[].revocation_date) should be ignored for idempotency checks.
              The timestamp O(revoked_certificates[].invalidity_date) will never be ignored.
            - Use this in combination with relative timestamps for these values to get idempotency.
        type: bool
        default: false

    return_content:
        description:
            - If set to V(true), will return the (current or generated) CRL's content as RV(crl).
        type: bool
        default: false

notes:
    - All ASN.1 TIME values should be specified following the YYYYMMDDHHMMSSZ pattern.
    - Date specified should be UTC. Minutes and seconds are mandatory.

seealso:
    - plugin: community.crypto.parse_serial
      plugin_type: filter
    - plugin: community.crypto.to_serial
      plugin_type: filter
'''

EXAMPLES = r'''
- name: Generate a CRL
  community.crypto.x509_crl:
    path: /etc/ssl/my-ca.crl
    privatekey_path: /etc/ssl/private/my-ca.pem
    issuer:
      CN: My CA
    last_update: "+0s"
    next_update: "+7d"
    revoked_certificates:
      - serial_number: 1234
        revocation_date: 20190331202428Z
        issuer:
          CN: My CA
      - serial_number: 2345
        revocation_date: 20191013152910Z
        reason: affiliation_changed
        invalidity_date: 20191001000000Z
      - path: /etc/ssl/crt/revoked-cert.pem
        revocation_date: 20191010010203Z
'''

RETURN = r'''
filename:
    description: Path to the generated CRL.
    returned: changed or success
    type: str
    sample: /path/to/my-ca.crl
backup_file:
    description: Name of backup file created.
    returned: changed and if O(backup) is V(true)
    type: str
    sample: /path/to/my-ca.crl.2019-03-09@11:22~
privatekey:
    description: Path to the private CA key.
    returned: changed or success
    type: str
    sample: /path/to/my-ca.pem
format:
    description:
        - Whether the CRL is in PEM format (V(pem)) or in DER format (V(der)).
    returned: success
    type: str
    sample: pem
    choices:
        - pem
        - der
issuer:
    description:
        - The CRL's issuer.
        - Note that for repeated values, only the last one will be returned.
        - See O(name_encoding) for how IDNs are handled.
    returned: success
    type: dict
    sample: {"organizationName": "Ansible", "commonName": "ca.example.com"}
issuer_ordered:
    description: The CRL's issuer as an ordered list of tuples.
    returned: success
    type: list
    elements: list
    sample: [["organizationName", "Ansible"], ["commonName": "ca.example.com"]]
last_update:
    description: The point in time from which this CRL can be trusted as ASN.1 TIME.
    returned: success
    type: str
    sample: 20190413202428Z
next_update:
    description: The point in time from which a new CRL will be issued and the client has to check for it as ASN.1 TIME.
    returned: success
    type: str
    sample: 20190413202428Z
digest:
    description: The signature algorithm used to sign the CRL.
    returned: success
    type: str
    sample: sha256WithRSAEncryption
revoked_certificates:
    description: List of certificates to be revoked.
    returned: success
    type: list
    elements: dict
    contains:
        serial_number:
            description:
                - Serial number of the certificate.
                - This return value is an B(integer). If you need the serial numbers as a colon-separated hex string,
                  such as C(11:22:33), you need to convert it to that form with P(community.crypto.to_serial#filter).
            type: int
            sample: 1234
        revocation_date:
            description: The point in time the certificate was revoked as ASN.1 TIME.
            type: str
            sample: 20190413202428Z
        issuer:
            description:
                - The certificate's issuer.
                - See O(name_encoding) for how IDNs are handled.
            type: list
            elements: str
            sample: ["DNS:ca.example.org"]
        issuer_critical:
            description: Whether the certificate issuer extension is critical.
            type: bool
            sample: false
        reason:
            description:
                - The value for the revocation reason extension.
            type: str
            sample: key_compromise
            choices:
                - unspecified
                - key_compromise
                - ca_compromise
                - affiliation_changed
                - superseded
                - cessation_of_operation
                - certificate_hold
                - privilege_withdrawn
                - aa_compromise
                - remove_from_crl
        reason_critical:
            description: Whether the revocation reason extension is critical.
            type: bool
            sample: false
        invalidity_date:
            description: |
                The point in time it was known/suspected that the private key was compromised
                or that the certificate otherwise became invalid as ASN.1 TIME.
            type: str
            sample: 20190413202428Z
        invalidity_date_critical:
            description: Whether the invalidity date extension is critical.
            type: bool
            sample: false
crl:
    description:
        - The (current or generated) CRL's content.
        - Will be the CRL itself if O(format) is V(pem), and Base64 of the
          CRL if O(format) is V(der).
    returned: if O(state) is V(present) and O(return_content) is V(true)
    type: str
'''


import base64
import os
import traceback

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.common.text.converters import to_native, to_text
from ansible.module_utils.common.validation import check_type_int, check_type_str

from ansible_collections.community.crypto.plugins.module_utils.serial import parse_serial
from ansible_collections.community.crypto.plugins.module_utils.version import LooseVersion

from ansible_collections.community.crypto.plugins.module_utils.io import (
    write_file,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.basic import (
    OpenSSLObjectError,
    OpenSSLBadPassphraseError,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.support import (
    OpenSSLObject,
    load_privatekey,
    load_certificate,
    parse_name_field,
    parse_ordered_name_field,
    get_relative_time_option,
    select_message_digest,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.cryptography_support import (
    cryptography_decode_name,
    cryptography_get_name,
    cryptography_key_needs_digest_for_signing,
    cryptography_name_to_oid,
    cryptography_oid_to_name,
    cryptography_serial_number_of_cert,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.cryptography_crl import (
    REVOCATION_REASON_MAP,
    TIMESTAMP_FORMAT,
    cryptography_decode_revoked_certificate,
    cryptography_dump_revoked,
    cryptography_get_signature_algorithm_oid_from_crl,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.pem import (
    identify_pem_format,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.module_backends.crl_info import (
    get_crl_info,
)

MINIMAL_CRYPTOGRAPHY_VERSION = '1.2'

CRYPTOGRAPHY_IMP_ERR = None
try:
    import cryptography
    from cryptography import x509
    from cryptography.hazmat.backends import default_backend
    from cryptography.hazmat.primitives.serialization import Encoding
    from cryptography.x509 import (
        CertificateRevocationListBuilder,
        RevokedCertificateBuilder,
        NameAttribute,
        Name,
    )
    CRYPTOGRAPHY_VERSION = LooseVersion(cryptography.__version__)
except ImportError:
    CRYPTOGRAPHY_IMP_ERR = traceback.format_exc()
    CRYPTOGRAPHY_FOUND = False
else:
    CRYPTOGRAPHY_FOUND = True


class CRLError(OpenSSLObjectError):
    pass


class CRL(OpenSSLObject):

    def __init__(self, module):
        super(CRL, self).__init__(
            module.params['path'],
            module.params['state'],
            module.params['force'],
            module.check_mode
        )

        self.format = module.params['format']

        self.update = module.params['crl_mode'] == 'update'
        self.ignore_timestamps = module.params['ignore_timestamps']
        self.return_content = module.params['return_content']
        self.name_encoding = module.params['name_encoding']
        self.serial_numbers_format = module.params['serial_numbers']
        self.crl_content = None

        self.privatekey_path = module.params['privatekey_path']
        self.privatekey_content = module.params['privatekey_content']
        if self.privatekey_content is not None:
            self.privatekey_content = self.privatekey_content.encode('utf-8')
        self.privatekey_passphrase = module.params['privatekey_passphrase']

        try:
            if module.params['issuer_ordered']:
                self.issuer_ordered = True
                self.issuer = parse_ordered_name_field(module.params['issuer_ordered'], 'issuer_ordered')
            else:
                self.issuer_ordered = False
                self.issuer = parse_name_field(module.params['issuer'], 'issuer')
        except (TypeError, ValueError) as exc:
            module.fail_json(msg=to_native(exc))

        self.last_update = get_relative_time_option(module.params['last_update'], 'last_update')
        self.next_update = get_relative_time_option(module.params['next_update'], 'next_update')

        self.digest = select_message_digest(module.params['digest'])
        if self.digest is None:
            raise CRLError('The digest "{0}" is not supported'.format(module.params['digest']))

        self.module = module

        self.revoked_certificates = []
        for i, rc in enumerate(module.params['revoked_certificates']):
            result = {
                'serial_number': None,
                'revocation_date': None,
                'issuer': None,
                'issuer_critical': False,
                'reason': None,
                'reason_critical': False,
                'invalidity_date': None,
                'invalidity_date_critical': False,
            }
            path_prefix = 'revoked_certificates[{0}].'.format(i)
            if rc['path'] is not None or rc['content'] is not None:
                # Load certificate from file or content
                try:
                    if rc['content'] is not None:
                        rc['content'] = rc['content'].encode('utf-8')
                    cert = load_certificate(rc['path'], content=rc['content'], backend='cryptography')
                    result['serial_number'] = cryptography_serial_number_of_cert(cert)
                except OpenSSLObjectError as e:
                    if rc['content'] is not None:
                        module.fail_json(
                            msg='Cannot parse certificate from {0}content: {1}'.format(path_prefix, to_native(e))
                        )
                    else:
                        module.fail_json(
                            msg='Cannot read certificate "{1}" from {0}path: {2}'.format(path_prefix, rc['path'], to_native(e))
                        )
            else:
                # Specify serial_number (and potentially issuer) directly
                result['serial_number'] = self._parse_serial_number(rc['serial_number'], i)
            # All other options
            if rc['issuer']:
                result['issuer'] = [cryptography_get_name(issuer, 'issuer') for issuer in rc['issuer']]
                result['issuer_critical'] = rc['issuer_critical']
            result['revocation_date'] = get_relative_time_option(
                rc['revocation_date'],
                path_prefix + 'revocation_date'
            )
            if rc['reason']:
                result['reason'] = REVOCATION_REASON_MAP[rc['reason']]
                result['reason_critical'] = rc['reason_critical']
            if rc['invalidity_date']:
                result['invalidity_date'] = get_relative_time_option(
                    rc['invalidity_date'],
                    path_prefix + 'invalidity_date'
                )
                result['invalidity_date_critical'] = rc['invalidity_date_critical']
            self.revoked_certificates.append(result)

        self.backup = module.params['backup']
        self.backup_file = None

        try:
            self.privatekey = load_privatekey(
                path=self.privatekey_path,
                content=self.privatekey_content,
                passphrase=self.privatekey_passphrase,
                backend='cryptography'
            )
        except OpenSSLBadPassphraseError as exc:
            raise CRLError(exc)

        self.crl = None
        try:
            with open(self.path, 'rb') as f:
                data = f.read()
            self.actual_format = 'pem' if identify_pem_format(data) else 'der'
            if self.actual_format == 'pem':
                self.crl = x509.load_pem_x509_crl(data, default_backend())
                if self.return_content:
                    self.crl_content = data
            else:
                self.crl = x509.load_der_x509_crl(data, default_backend())
                if self.return_content:
                    self.crl_content = base64.b64encode(data)
        except Exception as dummy:
            self.crl_content = None
            self.actual_format = self.format
            data = None

        self.diff_after = self.diff_before = self._get_info(data)

    def _parse_serial_number(self, value, index):
        if self.serial_numbers_format == 'integer':
            try:
                return check_type_int(value)
            except TypeError as exc:
                self.module.fail_json(msg='Error while parsing revoked_certificates[{idx}].serial_number as an integer: {exc}'.format(
                    idx=index + 1,
                    exc=to_native(exc),
                ))
        if self.serial_numbers_format == 'hex-octets':
            try:
                return parse_serial(check_type_str(value))
            except (TypeError, ValueError) as exc:
                self.module.fail_json(msg='Error while parsing revoked_certificates[{idx}].serial_number as an colon-separated hex octet string: {exc}'.format(
                    idx=index + 1,
                    exc=to_native(exc),
                ))
        raise RuntimeError('Unexpected value %s of serial_numbers' % (self.serial_numbers_format, ))

    def _get_info(self, data):
        if data is None:
            return dict()
        try:
            result = get_crl_info(self.module, data)
            result['can_parse_crl'] = True
            return result
        except Exception as exc:
            return dict(can_parse_crl=False)

    def remove(self):
        if self.backup:
            self.backup_file = self.module.backup_local(self.path)
        super(CRL, self).remove(self.module)

    def _compress_entry(self, entry):
        issuer = None
        if entry['issuer'] is not None:
            # Normalize to IDNA. If this is used-provided, it was already converted to
            # IDNA (by cryptography_get_name) and thus the `idna` library is present.
            # If this is coming from cryptography and isn't already in IDNA (i.e. ascii),
            # cryptography < 2.1 must be in use, which depends on `idna`. So this should
            # not require `idna` except if it was already used by code earlier during
            # this invocation.
            issuer = tuple(cryptography_decode_name(issuer, idn_rewrite='idna') for issuer in entry['issuer'])
        if self.ignore_timestamps:
            # Throw out revocation_date
            return (
                entry['serial_number'],
                issuer,
                entry['issuer_critical'],
                entry['reason'],
                entry['reason_critical'],
                entry['invalidity_date'],
                entry['invalidity_date_critical'],
            )
        else:
            return (
                entry['serial_number'],
                entry['revocation_date'],
                issuer,
                entry['issuer_critical'],
                entry['reason'],
                entry['reason_critical'],
                entry['invalidity_date'],
                entry['invalidity_date_critical'],
            )

    def check(self, module, perms_required=True, ignore_conversion=True):
        """Ensure the resource is in its desired state."""

        state_and_perms = super(CRL, self).check(self.module, perms_required)

        if not state_and_perms:
            return False

        if self.crl is None:
            return False

        if self.last_update != self.crl.last_update and not self.ignore_timestamps:
            return False
        if self.next_update != self.crl.next_update and not self.ignore_timestamps:
            return False
        if cryptography_key_needs_digest_for_signing(self.privatekey):
            if self.crl.signature_hash_algorithm is None or self.digest.name != self.crl.signature_hash_algorithm.name:
                return False
        else:
            if self.crl.signature_hash_algorithm is not None:
                return False

        want_issuer = [(cryptography_name_to_oid(entry[0]), entry[1]) for entry in self.issuer]
        is_issuer = [(sub.oid, sub.value) for sub in self.crl.issuer]
        if not self.issuer_ordered:
            want_issuer = set(want_issuer)
            is_issuer = set(is_issuer)
        if want_issuer != is_issuer:
            return False

        old_entries = [self._compress_entry(cryptography_decode_revoked_certificate(cert)) for cert in self.crl]
        new_entries = [self._compress_entry(cert) for cert in self.revoked_certificates]
        if self.update:
            # We do not simply use a set so that duplicate entries are treated correctly
            for entry in new_entries:
                try:
                    old_entries.remove(entry)
                except ValueError:
                    return False
        else:
            if old_entries != new_entries:
                return False

        if self.format != self.actual_format and not ignore_conversion:
            return False

        return True

    def _generate_crl(self):
        backend = default_backend()
        crl = CertificateRevocationListBuilder()

        try:
            crl = crl.issuer_name(Name([
                NameAttribute(cryptography_name_to_oid(entry[0]), to_text(entry[1]))
                for entry in self.issuer
            ]))
        except ValueError as e:
            raise CRLError(e)

        crl = crl.last_update(self.last_update)
        crl = crl.next_update(self.next_update)

        if self.update and self.crl:
            new_entries = set([self._compress_entry(entry) for entry in self.revoked_certificates])
            for entry in self.crl:
                decoded_entry = self._compress_entry(cryptography_decode_revoked_certificate(entry))
                if decoded_entry not in new_entries:
                    crl = crl.add_revoked_certificate(entry)
        for entry in self.revoked_certificates:
            revoked_cert = RevokedCertificateBuilder()
            revoked_cert = revoked_cert.serial_number(entry['serial_number'])
            revoked_cert = revoked_cert.revocation_date(entry['revocation_date'])
            if entry['issuer'] is not None:
                revoked_cert = revoked_cert.add_extension(
                    x509.CertificateIssuer(entry['issuer']),
                    entry['issuer_critical']
                )
            if entry['reason'] is not None:
                revoked_cert = revoked_cert.add_extension(
                    x509.CRLReason(entry['reason']),
                    entry['reason_critical']
                )
            if entry['invalidity_date'] is not None:
                revoked_cert = revoked_cert.add_extension(
                    x509.InvalidityDate(entry['invalidity_date']),
                    entry['invalidity_date_critical']
                )
            crl = crl.add_revoked_certificate(revoked_cert.build(backend))

        digest = None
        if cryptography_key_needs_digest_for_signing(self.privatekey):
            digest = self.digest
        self.crl = crl.sign(self.privatekey, digest, backend=backend)
        if self.format == 'pem':
            return self.crl.public_bytes(Encoding.PEM)
        else:
            return self.crl.public_bytes(Encoding.DER)

    def generate(self):
        result = None
        if not self.check(self.module, perms_required=False, ignore_conversion=True) or self.force:
            result = self._generate_crl()
        elif not self.check(self.module, perms_required=False, ignore_conversion=False) and self.crl:
            if self.format == 'pem':
                result = self.crl.public_bytes(Encoding.PEM)
            else:
                result = self.crl.public_bytes(Encoding.DER)

        if result is not None:
            self.diff_after = self._get_info(result)
            if self.return_content:
                if self.format == 'pem':
                    self.crl_content = result
                else:
                    self.crl_content = base64.b64encode(result)
            if self.backup:
                self.backup_file = self.module.backup_local(self.path)
            write_file(self.module, result)
            self.changed = True

        file_args = self.module.load_file_common_arguments(self.module.params)
        if self.module.check_file_absent_if_check_mode(file_args['path']):
            self.changed = True
        elif self.module.set_fs_attributes_if_different(file_args, False):
            self.changed = True

    def dump(self, check_mode=False):
        result = {
            'changed': self.changed,
            'filename': self.path,
            'privatekey': self.privatekey_path,
            'format': self.format,
            'last_update': None,
            'next_update': None,
            'digest': None,
            'issuer_ordered': None,
            'issuer': None,
            'revoked_certificates': [],
        }
        if self.backup_file:
            result['backup_file'] = self.backup_file

        if check_mode:
            result['last_update'] = self.last_update.strftime(TIMESTAMP_FORMAT)
            result['next_update'] = self.next_update.strftime(TIMESTAMP_FORMAT)
            # result['digest'] = cryptography_oid_to_name(self.crl.signature_algorithm_oid)
            result['digest'] = self.module.params['digest']
            result['issuer_ordered'] = self.issuer
            result['issuer'] = {}
            for k, v in self.issuer:
                result['issuer'][k] = v
            result['revoked_certificates'] = []
            for entry in self.revoked_certificates:
                result['revoked_certificates'].append(cryptography_dump_revoked(entry, idn_rewrite=self.name_encoding))
        elif self.crl:
            result['last_update'] = self.crl.last_update.strftime(TIMESTAMP_FORMAT)
            result['next_update'] = self.crl.next_update.strftime(TIMESTAMP_FORMAT)
            result['digest'] = cryptography_oid_to_name(cryptography_get_signature_algorithm_oid_from_crl(self.crl))
            issuer = []
            for attribute in self.crl.issuer:
                issuer.append([cryptography_oid_to_name(attribute.oid), attribute.value])
            result['issuer_ordered'] = issuer
            result['issuer'] = {}
            for k, v in issuer:
                result['issuer'][k] = v
            result['revoked_certificates'] = []
            for cert in self.crl:
                entry = cryptography_decode_revoked_certificate(cert)
                result['revoked_certificates'].append(cryptography_dump_revoked(entry, idn_rewrite=self.name_encoding))

        if self.return_content:
            result['crl'] = self.crl_content

        result['diff'] = dict(
            before=self.diff_before,
            after=self.diff_after,
        )
        return result


def main():
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(type='str', default='present', choices=['present', 'absent']),
            crl_mode=dict(
                type='str',
                # default='generate',
                choices=['generate', 'update'],
            ),
            mode=dict(
                type='str',
                # default='generate',
                choices=['generate', 'update'],
                removed_in_version='3.0.0',
                removed_from_collection='community.crypto',
            ),
            force=dict(type='bool', default=False),
            backup=dict(type='bool', default=False),
            path=dict(type='path', required=True),
            format=dict(type='str', default='pem', choices=['pem', 'der']),
            privatekey_path=dict(type='path'),
            privatekey_content=dict(type='str', no_log=True),
            privatekey_passphrase=dict(type='str', no_log=True),
            issuer=dict(type='dict'),
            issuer_ordered=dict(type='list', elements='dict'),
            last_update=dict(type='str', default='+0s'),
            next_update=dict(type='str'),
            digest=dict(type='str', default='sha256'),
            ignore_timestamps=dict(type='bool', default=False),
            return_content=dict(type='bool', default=False),
            revoked_certificates=dict(
                type='list',
                elements='dict',
                options=dict(
                    path=dict(type='path'),
                    content=dict(type='str'),
                    serial_number=dict(type='raw'),
                    revocation_date=dict(type='str', default='+0s'),
                    issuer=dict(type='list', elements='str'),
                    issuer_critical=dict(type='bool', default=False),
                    reason=dict(
                        type='str',
                        choices=[
                            'unspecified', 'key_compromise', 'ca_compromise', 'affiliation_changed',
                            'superseded', 'cessation_of_operation', 'certificate_hold',
                            'privilege_withdrawn', 'aa_compromise', 'remove_from_crl'
                        ]
                    ),
                    reason_critical=dict(type='bool', default=False),
                    invalidity_date=dict(type='str'),
                    invalidity_date_critical=dict(type='bool', default=False),
                ),
                required_one_of=[['path', 'content', 'serial_number']],
                mutually_exclusive=[['path', 'content', 'serial_number']],
            ),
            name_encoding=dict(type='str', default='ignore', choices=['ignore', 'idna', 'unicode']),
            serial_numbers=dict(type='str', default='integer', choices=['integer', 'hex-octets']),
        ),
        required_if=[
            ('state', 'present', ['privatekey_path', 'privatekey_content'], True),
            ('state', 'present', ['issuer', 'issuer_ordered'], True),
            ('state', 'present', ['next_update', 'revoked_certificates'], False),
        ],
        mutually_exclusive=(
            ['privatekey_path', 'privatekey_content'],
            ['issuer', 'issuer_ordered'],
        ),
        supports_check_mode=True,
        add_file_common_args=True,
    )

    if module.params['mode']:
        if module.params['crl_mode']:
            module.fail_json('You cannot use both `mode` and `crl_mode`. Use `crl_mode`.')
        module.params['crl_mode'] = module.params['mode']
    # TODO: in 3.0.0, once the option `mode` has been removed, remove this:
    module.params.pop('mode', None)
    # From then on, `mode` will be the file mode of the CRL file

    if not CRYPTOGRAPHY_FOUND:
        module.fail_json(msg=missing_required_lib('cryptography >= {0}'.format(MINIMAL_CRYPTOGRAPHY_VERSION)),
                         exception=CRYPTOGRAPHY_IMP_ERR)

    try:
        crl = CRL(module)

        if module.params['state'] == 'present':
            if module.check_mode:
                result = crl.dump(check_mode=True)
                result['changed'] = module.params['force'] or not crl.check(module) or not crl.check(module, ignore_conversion=False)
                module.exit_json(**result)

            crl.generate()
        else:
            if module.check_mode:
                result = crl.dump(check_mode=True)
                result['changed'] = os.path.exists(module.params['path'])
                module.exit_json(**result)

            crl.remove()

        result = crl.dump()
        module.exit_json(**result)
    except OpenSSLObjectError as exc:
        module.fail_json(msg=to_native(exc))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: roles_info
short_description: Information module for Roles
description:
- Get all Roles.
- Get all roles for the Cisco DNA Center system.
version_added: '6.7.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for User and Roles GetRolesAPI
  description: Complete reference of the GetRolesAPI API.
  link: https://developer.cisco.com/docs/dna-center/#!get-roles-api
notes:
  - SDK Method used are
    user_and_roles.UserandRoles.get_roles_ap_i,

  - Paths used are
    get /dna/system/api/v1/roles,

"""

EXAMPLES = r"""
- name: Get all Roles
  cisco.dnac.roles_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "roles": [
        {
          "resourceTypes": [
            {
              "operations": [
                "string"
              ],
              "type": "string"
            }
          ],
          "meta": {
            "createdBy": "string",
            "created": "string",
            "lastModified": "string"
          },
          "roleId": "string",
          "name": "string",
          "description": "string",
          "type": "string"
        }
      ]
    }
"""



# -*- coding: utf-8 -*-

# Copyright (c) 2017, Eike Frost <ei@kefro.st>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):

    # Standard documentation fragment
    DOCUMENTATION = r'''
options:
    auth_keycloak_url:
        description:
            - URL to the Keycloak instance.
        type: str
        required: true
        aliases:
          - url

    auth_client_id:
        description:
            - OpenID Connect C(client_id) to authenticate to the API with.
        type: str
        default: admin-cli

    auth_realm:
        description:
            - Keycloak realm name to authenticate to for API access.
        type: str

    auth_client_secret:
        description:
            - Client Secret to use in conjunction with O(auth_client_id) (if required).
        type: str

    auth_username:
        description:
            - Username to authenticate for API access with.
        type: str
        aliases:
          - username

    auth_password:
        description:
            - Password to authenticate for API access with.
        type: str
        aliases:
          - password

    token:
        description:
            - Authentication token for Keycloak API.
        type: str
        version_added: 3.0.0

    validate_certs:
        description:
            - Verify TLS certificates (do not disable this in production).
        type: bool
        default: true

    connection_timeout:
        description:
            - Controls the HTTP connections timeout period (in seconds) to Keycloak API.
        type: int
        default: 10
        version_added: 4.5.0

    http_agent:
        description:
            - Configures the HTTP User-Agent header.
        type: str
        default: Ansible
        version_added: 5.4.0
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: repository_info
short_description: Information module for Repository
description:
- Get all Repository.
- Get Repository by name.
- Get a specific repository identified by the name passed in the URL.
- This will get the full list of repository definitions on the system.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  repositoryName:
    description:
    - RepositoryName path parameter. Unique name for a repository.
    type: str
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Repository
  description: Complete reference of the Repository API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!repository-openapi
notes:
  - SDK Method used are
    repository.Repository.get_repositories,
    repository.Repository.get_repository,

  - Paths used are
    get /api/v1/repository,
    get /api/v1/repository/{repositoryName},

"""

EXAMPLES = r"""
- name: Get all Repository
  cisco.ise.repository_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
  register: result

- name: Get Repository by name
  cisco.ise.repository_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    repositoryName: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "name": "string",
      "protocol": "string",
      "path": "string",
      "password": "string",
      "serverName": "string",
      "userName": "string",
      "enablePki": true
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: devices_appliance_uplinks_settings
short_description: Resource module for devices _appliance _uplinks _settings
description:
- Manage operation update of the resource devices _appliance _uplinks _settings.
- Update the uplink settings for an MX appliance.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  interfaces:
    description: Interface settings.
    suboptions:
      wan1:
        description: WAN 1 settings.
        suboptions:
          enabled:
            description: Enable or disable the interface.
            type: bool
          pppoe:
            description: Configuration options for PPPoE.
            suboptions:
              authentication:
                description: Settings for PPPoE Authentication.
                suboptions:
                  enabled:
                    description: Whether PPPoE authentication is enabled.
                    type: bool
                  password:
                    description: Password for PPPoE authentication. This parameter is
                      not returned.
                    type: str
                  username:
                    description: Username for PPPoE authentication.
                    type: str
                type: dict
              enabled:
                description: Whether PPPoE is enabled.
                type: bool
            type: dict
          svis:
            description: SVI settings by protocol.
            suboptions:
              ipv4:
                description: IPv4 settings for static/dynamic mode.
                suboptions:
                  address:
                    description: IP address and subnet mask when in static mode.
                    type: str
                  assignmentMode:
                    description: The assignment mode for this SVI. Applies only when
                      PPPoE is disabled.
                    type: str
                  gateway:
                    description: Gateway IP address when in static mode.
                    type: str
                  nameservers:
                    description: The nameserver settings for this SVI.
                    suboptions:
                      addresses:
                        description: Up to 2 nameserver addresses to use, ordered in
                          priority from highest to lowest priority.
                        elements: str
                        type: list
                    type: dict
                type: dict
              ipv6:
                description: IPv6 settings for static/dynamic mode.
                suboptions:
                  address:
                    description: Static address that will override the one(s) received
                      by SLAAC.
                    type: str
                  assignmentMode:
                    description: The assignment mode for this SVI. Applies only when
                      PPPoE is disabled.
                    type: str
                  gateway:
                    description: Static gateway that will override the one received
                      by autoconf.
                    type: str
                  nameservers:
                    description: The nameserver settings for this SVI.
                    suboptions:
                      addresses:
                        description: Up to 2 nameserver addresses to use, ordered in
                          priority from highest to lowest priority.
                        elements: str
                        type: list
                    type: dict
                type: dict
            type: dict
          vlanTagging:
            description: VLAN tagging settings.
            suboptions:
              enabled:
                description: Whether VLAN tagging is enabled.
                type: bool
              vlanId:
                description: The ID of the VLAN to use for VLAN tagging.
                type: int
            type: dict
        type: dict
      wan2:
        description: WAN 2 settings.
        suboptions:
          enabled:
            description: Enable or disable the interface.
            type: bool
          pppoe:
            description: Configuration options for PPPoE.
            suboptions:
              authentication:
                description: Settings for PPPoE Authentication.
                suboptions:
                  enabled:
                    description: Whether PPPoE authentication is enabled.
                    type: bool
                  password:
                    description: Password for PPPoE authentication. This parameter is
                      not returned.
                    type: str
                  username:
                    description: Username for PPPoE authentication.
                    type: str
                type: dict
              enabled:
                description: Whether PPPoE is enabled.
                type: bool
            type: dict
          svis:
            description: SVI settings by protocol.
            suboptions:
              ipv4:
                description: IPv4 settings for static/dynamic mode.
                suboptions:
                  address:
                    description: IP address and subnet mask when in static mode.
                    type: str
                  assignmentMode:
                    description: The assignment mode for this SVI. Applies only when
                      PPPoE is disabled.
                    type: str
                  gateway:
                    description: Gateway IP address when in static mode.
                    type: str
                  nameservers:
                    description: The nameserver settings for this SVI.
                    suboptions:
                      addresses:
                        description: Up to 2 nameserver addresses to use, ordered in
                          priority from highest to lowest priority.
                        elements: str
                        type: list
                    type: dict
                type: dict
              ipv6:
                description: IPv6 settings for static/dynamic mode.
                suboptions:
                  address:
                    description: Static address that will override the one(s) received
                      by SLAAC.
                    type: str
                  assignmentMode:
                    description: The assignment mode for this SVI. Applies only when
                      PPPoE is disabled.
                    type: str
                  gateway:
                    description: Static gateway that will override the one received
                      by autoconf.
                    type: str
                  nameservers:
                    description: The nameserver settings for this SVI.
                    suboptions:
                      addresses:
                        description: Up to 2 nameserver addresses to use, ordered in
                          priority from highest to lowest priority.
                        elements: str
                        type: list
                    type: dict
                type: dict
            type: dict
          vlanTagging:
            description: VLAN tagging settings.
            suboptions:
              enabled:
                description: Whether VLAN tagging is enabled.
                type: bool
              vlanId:
                description: The ID of the VLAN to use for VLAN tagging.
                type: int
            type: dict
        type: dict
    type: dict
  serial:
    description: Serial path parameter.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for appliance updateDeviceApplianceUplinksSettings
  description: Complete reference of the updateDeviceApplianceUplinksSettings API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-device-appliance-uplinks-settings
notes:
  - SDK Method used are
    appliance.Appliance.update_device_appliance_uplinks_settings,

  - Paths used are
    put /devices/{serial}/appliance/uplinks/settings,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.devices_appliance_uplinks_settings:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    interfaces:
      wan1:
        enabled: true
        pppoe:
          authentication:
            enabled: true
            password: password
            username: username
          enabled: true
        svis:
          ipv4:
            address: 9.10.11.10/16
            assignmentMode: static
            gateway: 13.14.15.16
            nameservers:
              addresses:
              - 1.2.3.4
          ipv6:
            address: 1:2:3::4
            assignmentMode: static
            gateway: 1:2:3::5
            nameservers:
              addresses:
              - 1001:4860:4860::8888
              - 1001:4860:4860::8844
        vlanTagging:
          enabled: true
          vlanId: 1
      wan2:
        enabled: true
        pppoe:
          authentication:
            enabled: true
            password: password
            username: username
          enabled: true
        svis:
          ipv4:
            address: 9.10.11.10/16
            assignmentMode: static
            gateway: 13.14.15.16
            nameservers:
              addresses:
              - 1.2.3.4
          ipv6:
            address: 1:2:3::4
            assignmentMode: static
            gateway: 1:2:3::5
            nameservers:
              addresses:
              - 1001:4860:4860::8888
              - 1001:4860:4860::8844
        vlanTagging:
          enabled: true
          vlanId: 1
    serial: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "wan1": {
        "enabled": true,
        "vlanTagging": {
          "enabled": true,
          "vlanId": 0
        },
        "svis": {
          "ipv4": {
            "assignmentMode": "string",
            "address": "string",
            "gateway": "string",
            "nameservers": {
              "addresses": [
                "string"
              ]
            }
          },
          "ipv6": {
            "assignmentMode": "string",
            "address": "string",
            "gateway": "string",
            "nameservers": {
              "addresses": [
                "string"
              ]
            }
          }
        },
        "pppoe": {
          "enabled": true,
          "authentication": {
            "enabled": true,
            "username": "string"
          }
        }
      },
      "wan2": {
        "enabled": true,
        "vlanTagging": {
          "enabled": true,
          "vlanId": 0
        },
        "svis": {
          "ipv4": {
            "assignmentMode": "string",
            "address": "string",
            "gateway": "string",
            "nameservers": {
              "addresses": [
                "string"
              ]
            }
          },
          "ipv6": {
            "assignmentMode": "string",
            "address": "string",
            "gateway": "string",
            "nameservers": {
              "addresses": [
                "string"
              ]
            }
          }
        },
        "pppoe": {
          "enabled": true,
          "authentication": {
            "enabled": true,
            "username": "string"
          }
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_server_profile
short_description: Server Profile configuration for Cisco Intersight
description:
  - Server Profile configuration for Cisco Intersight.
  - Used to configure Server Profiles with assigned servers and server policies.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Server Profile.
      - The name must be between 1 and 62 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  target_platform:
    description:
      - The platform for which the server profile is applicable.
      - Can either be a server that is operating in Standalone mode or which is attached to a Fabric Interconnect (FIAttached) managed by Intersight.
    type: str
    choices: [Standalone, FIAttached]
    default: Standalone
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
    default: []
  description:
    description:
      - The user-defined description of the Server Profile.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
    default: ''
  assigned_server:
    description:
      - Managed Obect ID (MOID) of assigned server.
      - Option can be omitted if user wishes to assign server later.
    type: str
  bios_policy:
    description:
      - Name of BIOS Policy to associate with this profile.
    type: str
  boot_order_policy:
    description:
      - Name of Boot Order Policy to associate with this profile.
    type: str
  certificate_policy:
    description:
      - Name of Certificate Policy to associate with this profile.
    type: str
  drive_security_policy:
    description:
      - Name of Drive Security Policy to associate with this profile.
    type: str
  firmware_policy:
    description:
      - Name of Firmware Policy to associate with this profile.
    type: str
  imc_access_policy:
    description:
      - Name of IMC Access Policy to associate with this profile.
    type: str
  ipmi_over_lan_policy:
    description:
      - Name of IPMI over LAN Policy to associate with this profile.
    type: str
  lan_connectivity_policy:
    description:
      - Name of LAN Connectivity Policy to associate with this profile.
    type: str
  local_user_policy:
    description:
      - Name of Local User Policy to associate with this profile.
    type: str
  ntp_policy:
    description:
      - Name of NTP Policy to associate with this profile.
    type: str
  san_connectivity_policy:
    description:
      - Name of SAN Connectivity Policy to associate with this profile.
    type: str
  serial_over_lan_policy:
    description:
      - Name of Serial over LAN Policy to associate with this profile.
    type: str
  snmp_policy:
    description:
      - Name of SNMP Policy to associate with this profile.
    type: str
  storage_policy:
    description:
      - Name of Storage Policy to associate with this profile.
    type: str
  syslog_policy:
    description:
      - Name of Syslog Policy to associate with this profile.
    type: str
  thermal_policy:
    description:
      - Name of Thermal Policy to associate with this profile.
    type: str
  virtual_kvm_policy:
    description:
      - Name of Virtual KVM Policy to associate with this profile.
    type: str
  virtual_media_policy:
    description:
      - Name of Virtual Media Policy to associate with this profile.
    type: str
author:
  - David Soper (@dsoper2)
  - Sid Nath (@SidNath21)
  - Tse Kai "Kevin" Chan (@BrightScale)
  - Soma Tummala (@SOMATUMMALA21)
'''

EXAMPLES = r'''
- name: Configure Server Profile
  cisco.intersight.intersight_server_profile:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: SP-Server1
    target_platform: FIAttached
    tags:
      - Key: Site
        Value: SJC02
    description: Profile for Server1
    assigned_server: 5e3b517d6176752d319a9999
    boot_order_policy: COS-Boot
    imc_access_policy: sjc02-d23-access
    lan_connectivity_policy: sjc02-d23-lan
    local_user_policy: guest-admin
    ntp_policy: lab-ntp
    storage_policy: storage
    virtual_media_policy: COS-VM

- name: Delete Server Profile
  cisco.intersight.intersight_server_profile:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: SP-Server1
    state: absent
'''

RETURN = r'''
api_repsonse:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "AssignedServer": {
            "Moid": "5e3b517d6176752d319a0881",
            "ObjectType": "compute.Blade",
        },
        "Name": "SP-IMM-6454-D23-1-1",
        "ObjectType": "server.Profile",
        "Tags": [
            {
                "Key": "Site",
                "Value": "SJC02"
            }
        ],
        "TargetPlatform": "FIAttached",
        "Type": "instance"
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def post_profile_to_policy(intersight, moid, resource_path, policy_name):
    options = {
        'http_method': 'get',
        'resource_path': resource_path,
        'query_params': {
            '$filter': "Name eq '" + policy_name + "'",
        },
    }
    response = intersight.call_api(**options)
    if response.get('Results'):
        # get expected policy moid from 1st list element
        expected_policy_moid = response['Results'][0]['Moid']
        actual_policy_moid = ''
        # check any current profiles and delete if needed
        options = {
            'http_method': 'get',
            'resource_path': resource_path,
            'query_params': {
                '$filter': "Profiles/any(t: t/Moid eq '" + moid + "')",
            },
        }
        response = intersight.call_api(**options)
        if response.get('Results'):
            # get actual moid from 1st list element
            actual_policy_moid = response['Results'][0]['Moid']
            if actual_policy_moid != expected_policy_moid:
                if not intersight.module.check_mode:
                    # delete the actual policy
                    options = {
                        'http_method': 'delete',
                        'resource_path': resource_path + '/' + actual_policy_moid + '/Profiles',
                        'moid': moid,
                    }
                    intersight.call_api(**options)
                actual_policy_moid = ''
        if not actual_policy_moid:
            if not intersight.module.check_mode:
                # post profile to the expected policy
                options = {
                    'http_method': 'post',
                    'resource_path': resource_path + '/' + expected_policy_moid + '/Profiles',
                    'body': [
                        {
                            'ObjectType': 'server.Profile',
                            'Moid': moid,
                        }
                    ]
                }
                intersight.call_api(**options)
            intersight.result['changed'] = True


def main():
    argument_spec = intersight_argument_spec
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        target_platform=dict(type='str', choices=['Standalone', 'FIAttached'], default='Standalone'),
        tags=dict(type='list', elements='dict', default=[]),
        description=dict(type='str', aliases=['descr'], default=''),
        assigned_server=dict(type='str'),
        bios_policy=dict(type='str'),
        boot_order_policy=dict(type='str'),
        certificate_policy=dict(type='str'),
        drive_security_policy=dict(type='str'),
        firmware_policy=dict(type='str'),
        imc_access_policy=dict(type='str'),
        ipmi_over_lan_policy=dict(type='str'),
        lan_connectivity_policy=dict(type='str'),
        local_user_policy=dict(type='str'),
        ntp_policy=dict(type='str'),
        san_connectivity_policy=dict(type='str'),
        serial_over_lan_policy=dict(type='str'),
        snmp_policy=dict(type='str'),
        storage_policy=dict(type='str'),
        syslog_policy=dict(type='str'),
        thermal_policy=dict(type='str'),
        virtual_kvm_policy=dict(type='str'),
        virtual_media_policy=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''
    #
    # Argument spec above, resource path, and API body should be the only code changed in this module
    #
    resource_path = '/server/Profiles'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
        'Tags': intersight.module.params['tags'],
        'Description': intersight.module.params['description'],
    }
    intersight.result['api_response'] = {}
    # Get assigned server information (if defined)
    if intersight.module.params['assigned_server']:
        intersight.get_resource(
            resource_path='/compute/PhysicalSummaries',
            query_params={
                '$filter': "Moid eq '" + intersight.module.params['assigned_server'] + "'",
            }
        )
        source_object_type = None
        if intersight.result['api_response'].get('SourceObjectType'):
            source_object_type = intersight.result['api_response']['SourceObjectType']
        intersight.api_body['AssignedServer'] = {
            'Moid': intersight.module.params['assigned_server'],
            'ObjectType': source_object_type,
        }
    if intersight.module.params['target_platform'] == 'FIAttached':
        intersight.api_body['TargetPlatform'] = intersight.module.params['target_platform']

    # Configure the profile
    moid = intersight.configure_policy_or_profile(resource_path=resource_path)

    if moid and intersight.module.params['bios_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/bios/Policies', policy_name=intersight.module.params['bios_policy'])
    if moid and intersight.module.params['boot_order_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/boot/PrecisionPolicies', policy_name=intersight.module.params['boot_order_policy'])
    if moid and intersight.module.params['certificate_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/security/CertificatePolicies', policy_name=intersight.module.params['certificate_policy'])
    if moid and intersight.module.params['drive_security_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/security/DriveSecurityPolicies', policy_name=intersight.module.params['drive_security_policy'])
    if moid and intersight.module.params['firmware_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/firmware/Policies', policy_name=intersight.module.params['firmware_policy'])
    if moid and intersight.module.params['imc_access_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/access/Policies', policy_name=intersight.module.params['imc_access_policy'])
    if moid and intersight.module.params['ipmi_over_lan_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/ipmi/Policies', policy_name=intersight.module.params['ipmi_over_lan_policy'])
    if moid and intersight.module.params['lan_connectivity_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/vnic/LanConnectivityPolicies', policy_name=intersight.module.params['lan_connectivity_policy'])
    if moid and intersight.module.params['local_user_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/iam/EndPointUserPolicies', policy_name=intersight.module.params['local_user_policy'])
    if moid and intersight.module.params['ntp_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/ntp/Policies', policy_name=intersight.module.params['ntp_policy'])
    if moid and intersight.module.params['san_connectivity_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/vnic/SanConnectivityPolicies', policy_name=intersight.module.params['san_connectivity_policy'])
    if moid and intersight.module.params['serial_over_lan_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/serial/Policies', policy_name=intersight.module.params['serial_over_lan_policy'])
    if moid and intersight.module.params['snmp_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/snmp/Policies', policy_name=intersight.module.params['snmp_policy'])
    if moid and intersight.module.params['storage_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/storage/StoragePolicies', policy_name=intersight.module.params['storage_policy'])
    if moid and intersight.module.params['syslog_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/syslog/Policies', policy_name=intersight.module.params['syslog_policy'])
    if moid and intersight.module.params['thermal_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/thermal/Policies', policy_name=intersight.module.params['thermal_policy'])
    if moid and intersight.module.params['virtual_kvm_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/kvm/Policies', policy_name=intersight.module.params['virtual_kvm_policy'])
    if moid and intersight.module.params['virtual_media_policy']:
        post_profile_to_policy(intersight, moid, resource_path='/vmedia/Policies', policy_name=intersight.module.params['virtual_media_policy'])

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: configuration_template
short_description: Resource module for Configuration Template
description:
- Manage operations update and delete of the resource Configuration Template.
- Deletes the template by its id.
- API to update a template.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  author:
    description: Author of template.
    type: str
  composite:
    description: Is it composite template.
    type: bool
  containingTemplates:
    description: Configuration Template's containingTemplates.
    elements: dict
    suboptions:
      composite:
        description: Is it composite template.
        type: bool
      description:
        description: Description of template.
        type: str
      deviceTypes:
        description: Configuration Template's deviceTypes.
        elements: dict
        suboptions:
          productFamily:
            description: Device family.
            type: str
          productSeries:
            description: Device series.
            type: str
          productType:
            description: Device type.
            type: str
        type: list
      id:
        description: UUID of template.
        type: str
      language:
        description: Template language (JINJA or VELOCITY).
        type: str
      name:
        description: Name of template.
        type: str
      projectName:
        description: Project name.
        type: str
      rollbackTemplateParams:
        description: Configuration Template's rollbackTemplateParams.
        elements: dict
        suboptions:
          binding:
            description: Bind to source.
            type: str
          customOrder:
            description: CustomOrder of template param.
            type: int
          dataType:
            description: Datatype of template param.
            type: str
          defaultValue:
            description: Default value of template param.
            type: str
          description:
            description: Description of template param.
            type: str
          displayName:
            description: Display name of param.
            type: str
          group:
            description: Group.
            type: str
          id:
            description: UUID of template param.
            type: str
          instructionText:
            description: Instruction text for param.
            type: str
          key:
            description: Key.
            type: str
          notParam:
            description: Is it not a variable.
            type: bool
          order:
            description: Order of template param.
            type: int
          paramArray:
            description: Is it an array.
            type: bool
          parameterName:
            description: Name of template param.
            type: str
          provider:
            description: Provider.
            type: str
          range:
            description: Configuration Template's range.
            elements: dict
            suboptions:
              id:
                description: UUID of range.
                type: str
              maxValue:
                description: Max value of range.
                type: int
              minValue:
                description: Min value of range.
                type: int
            type: list
          required:
            description: Is param required.
            type: bool
          selection:
            description: Configuration Template's selection.
            suboptions:
              defaultSelectedValues:
                description: Default selection values.
                elements: str
                type: list
              id:
                description: UUID of selection.
                type: str
              selectionType:
                description: Type of selection(SINGLE_SELECT or MULTI_SELECT).
                type: str
              selectionValues:
                description: Selection values.
                type: dict
            type: dict
        type: list
      tags:
        description: Configuration Template's tags.
        elements: dict
        suboptions:
          id:
            description: UUID of tag.
            type: str
          name:
            description: Name of tag.
            type: str
        type: list
      templateContent:
        description: Template content.
        type: str
      templateParams:
        description: Configuration Template's templateParams.
        elements: dict
        suboptions:
          binding:
            description: Bind to source.
            type: str
          customOrder:
            description: CustomOrder of template param.
            type: int
          dataType:
            description: Datatype of template param.
            type: str
          defaultValue:
            description: Default value of template param.
            type: str
          description:
            description: Description of template param.
            type: str
          displayName:
            description: Display name of param.
            type: str
          group:
            description: Group.
            type: str
          id:
            description: UUID of template param.
            type: str
          instructionText:
            description: Instruction text for param.
            type: str
          key:
            description: Key.
            type: str
          notParam:
            description: Is it not a variable.
            type: bool
          order:
            description: Order of template param.
            type: int
          paramArray:
            description: Is it an array.
            type: bool
          parameterName:
            description: Name of template param.
            type: str
          provider:
            description: Provider.
            type: str
          range:
            description: Configuration Template's range.
            elements: dict
            suboptions:
              id:
                description: UUID of range.
                type: str
              maxValue:
                description: Max value of range.
                type: int
              minValue:
                description: Min value of range.
                type: int
            type: list
          required:
            description: Is param required.
            type: bool
          selection:
            description: Configuration Template's selection.
            suboptions:
              defaultSelectedValues:
                description: Default selection values.
                elements: str
                type: list
              id:
                description: UUID of selection.
                type: str
              selectionType:
                description: Type of selection(SINGLE_SELECT or MULTI_SELECT).
                type: str
              selectionValues:
                description: Selection values.
                type: dict
            type: dict
        type: list
      version:
        description: Current version of template.
        type: str
    type: list
  createTime:
    description: Create time of template.
    type: int
  customParamsOrder:
    description: Custom Params Order.
    type: bool
  description:
    description: Description of template.
    type: str
  deviceTypes:
    description: Configuration Template's deviceTypes.
    elements: dict
    suboptions:
      productFamily:
        description: Device family.
        type: str
      productSeries:
        description: Device series.
        type: str
      productType:
        description: Device type.
        type: str
    type: list
  failurePolicy:
    description: Define failure policy if template provisioning fails.
    type: str
  id:
    description: UUID of template.
    type: str
  language:
    description: Template language (JINJA or VELOCITY).
    type: str
  lastUpdateTime:
    description: Update time of template.
    type: int
  latestVersionTime:
    description: Latest versioned template time.
    type: int
  name:
    description: Name of template.
    type: str
  parentTemplateId:
    description: Parent templateID.
    type: str
  projectId:
    description: Project UUID.
    type: str
  projectName:
    description: Project name.
    type: str
  rollbackTemplateContent:
    description: Rollback template content.
    type: str
  rollbackTemplateParams:
    description: Configuration Template's rollbackTemplateParams.
    elements: dict
    suboptions:
      binding:
        description: Bind to source.
        type: str
      customOrder:
        description: CustomOrder of template param.
        type: int
      dataType:
        description: Datatype of template param.
        type: str
      defaultValue:
        description: Default value of template param.
        type: str
      description:
        description: Description of template param.
        type: str
      displayName:
        description: Display name of param.
        type: str
      group:
        description: Group.
        type: str
      id:
        description: UUID of template param.
        type: str
      instructionText:
        description: Instruction text for param.
        type: str
      key:
        description: Key.
        type: str
      notParam:
        description: Is it not a variable.
        type: bool
      order:
        description: Order of template param.
        type: int
      paramArray:
        description: Is it an array.
        type: bool
      parameterName:
        description: Name of template param.
        type: str
      provider:
        description: Provider.
        type: str
      range:
        description: Configuration Template's range.
        elements: dict
        suboptions:
          id:
            description: UUID of range.
            type: str
          maxValue:
            description: Max value of range.
            type: int
          minValue:
            description: Min value of range.
            type: int
        type: list
      required:
        description: Is param required.
        type: bool
      selection:
        description: Configuration Template's selection.
        suboptions:
          defaultSelectedValues:
            description: Default selection values.
            elements: str
            type: list
          id:
            description: UUID of selection.
            type: str
          selectionType:
            description: Type of selection(SINGLE_SELECT or MULTI_SELECT).
            type: str
          selectionValues:
            description: Selection values.
            type: dict
        type: dict
    type: list
  softwareType:
    description: Applicable device software type.
    type: str
  softwareVariant:
    description: Applicable device software variant.
    type: str
  softwareVersion:
    description: Applicable device software version.
    type: str
  tags:
    description: Configuration Template's tags.
    elements: dict
    suboptions:
      id:
        description: UUID of tag.
        type: str
      name:
        description: Name of tag.
        type: str
    type: list
  templateContent:
    description: Template content.
    type: str
  templateId:
    description: TemplateId path parameter. TemplateId(UUID) of template to be deleted.
    type: str
  templateParams:
    description: Configuration Template's templateParams.
    elements: dict
    suboptions:
      binding:
        description: Bind to source.
        type: str
      customOrder:
        description: CustomOrder of template param.
        type: int
      dataType:
        description: Datatype of template param.
        type: str
      defaultValue:
        description: Default value of template param.
        type: str
      description:
        description: Description of template param.
        type: str
      displayName:
        description: Display name of param.
        type: str
      group:
        description: Group.
        type: str
      id:
        description: UUID of template param.
        type: str
      instructionText:
        description: Instruction text for param.
        type: str
      key:
        description: Key.
        type: str
      notParam:
        description: Is it not a variable.
        type: bool
      order:
        description: Order of template param.
        type: int
      paramArray:
        description: Is it an array.
        type: bool
      parameterName:
        description: Name of template param.
        type: str
      provider:
        description: Provider.
        type: str
      range:
        description: Configuration Template's range.
        elements: dict
        suboptions:
          id:
            description: UUID of range.
            type: str
          maxValue:
            description: Max value of range.
            type: int
          minValue:
            description: Min value of range.
            type: int
        type: list
      required:
        description: Is param required.
        type: bool
      selection:
        description: Configuration Template's selection.
        suboptions:
          defaultSelectedValues:
            description: Default selection values.
            elements: str
            type: list
          id:
            description: UUID of selection.
            type: str
          selectionType:
            description: Type of selection(SINGLE_SELECT or MULTI_SELECT).
            type: str
          selectionValues:
            description: Selection values.
            type: dict
        type: dict
    type: list
  validationErrors:
    description: Configuration Template's validationErrors.
    suboptions:
      rollbackTemplateErrors:
        description: Validation or design conflicts errors of rollback template.
        type: dict
      templateErrors:
        description: Validation or design conflicts errors.
        type: dict
      templateId:
        description: UUID of template.
        type: str
      templateVersion:
        description: Current version of template.
        type: str
    type: dict
  version:
    description: Current version of template.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Configuration Templates DeletesTheTemplate
  description: Complete reference of the DeletesTheTemplate API.
  link: https://developer.cisco.com/docs/dna-center/#!deletes-the-template
- name: Cisco DNA Center documentation for Configuration Templates UpdateTemplate
  description: Complete reference of the UpdateTemplate API.
  link: https://developer.cisco.com/docs/dna-center/#!update-template
notes:
  - SDK Method used are
    configuration_templates.ConfigurationTemplates.deletes_the_template,
    configuration_templates.ConfigurationTemplates.update_template,

  - Paths used are
    delete /dna/intent/api/v1/template-programmer/template/{templateId},
    put /dna/intent/api/v1/template-programmer/template,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.configuration_template:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    author: string
    composite: true
    containingTemplates:
    - composite: true
      description: string
      deviceTypes:
      - productFamily: string
        productSeries: string
        productType: string
      id: string
      language: string
      name: string
      projectName: string
      rollbackTemplateParams:
      - binding: string
        customOrder: 0
        dataType: string
        defaultValue: string
        description: string
        displayName: string
        group: string
        id: string
        instructionText: string
        key: string
        notParam: true
        order: 0
        paramArray: true
        parameterName: string
        provider: string
        range:
        - id: string
          maxValue: 0
          minValue: 0
        required: true
        selection:
          defaultSelectedValues:
          - string
          id: string
          selectionType: string
          selectionValues: {}
      tags:
      - id: string
        name: string
      templateContent: string
      templateParams:
      - binding: string
        customOrder: 0
        dataType: string
        defaultValue: string
        description: string
        displayName: string
        group: string
        id: string
        instructionText: string
        key: string
        notParam: true
        order: 0
        paramArray: true
        parameterName: string
        provider: string
        range:
        - id: string
          maxValue: 0
          minValue: 0
        required: true
        selection:
          defaultSelectedValues:
          - string
          id: string
          selectionType: string
          selectionValues: {}
      version: string
    createTime: 0
    customParamsOrder: true
    description: string
    deviceTypes:
    - productFamily: string
      productSeries: string
      productType: string
    failurePolicy: string
    id: string
    language: string
    lastUpdateTime: 0
    latestVersionTime: 0
    name: string
    parentTemplateId: string
    projectId: string
    projectName: string
    rollbackTemplateContent: string
    rollbackTemplateParams:
    - binding: string
      customOrder: 0
      dataType: string
      defaultValue: string
      description: string
      displayName: string
      group: string
      id: string
      instructionText: string
      key: string
      notParam: true
      order: 0
      paramArray: true
      parameterName: string
      provider: string
      range:
      - id: string
        maxValue: 0
        minValue: 0
      required: true
      selection:
        defaultSelectedValues:
        - string
        id: string
        selectionType: string
        selectionValues: {}
    softwareType: string
    softwareVariant: string
    softwareVersion: string
    tags:
    - id: string
      name: string
    templateContent: string
    templateParams:
    - binding: string
      customOrder: 0
      dataType: string
      defaultValue: string
      description: string
      displayName: string
      group: string
      id: string
      instructionText: string
      key: string
      notParam: true
      order: 0
      paramArray: true
      parameterName: string
      provider: string
      range:
      - id: string
        maxValue: 0
        minValue: 0
      required: true
      selection:
        defaultSelectedValues:
        - string
        id: string
        selectionType: string
        selectionValues: {}
    validationErrors:
      rollbackTemplateErrors: {}
      templateErrors: {}
      templateId: string
      templateVersion: string
    version: string

- name: Delete by id
  cisco.dnac.configuration_template:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    templateId: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""
